using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;
using System.Windows.Shapes;

namespace Softix.Drawing.Diagram.Wpf.Controls
{
	public sealed class ArrowDecorator : Decorator
	{
		private Polyline ln;
		private VisualCollection children;

		public ArrowDecorator()
		{
			this.children = new VisualCollection(this);

			this.ln = new Polyline();
			this.children.Add(this.ln);

			this.ClipToBounds = false;
		}

		protected override Size ArrangeOverride(Size arrangeSize)
		{
			Size finalSize = base.ArrangeOverride(arrangeSize);

			// this.x.Arrange(new Rect(0, 0, finalSize.Width, finalSize.Height));
			Line source = (this.Child as Line);
			if (source != null)
			{
				Point targetPoint = new Point(source.X1, source.Y1);
				Point sourcePoint = new Point(source.X2, source.Y2);
				if (sourcePoint == targetPoint)
					return finalSize;

				double l = Math.Sqrt(Math.Pow(sourcePoint.X - targetPoint.X, 2) + Math.Pow(sourcePoint.Y - targetPoint.Y, 2));
				double xd = (this.ArrowSize * (targetPoint.X - sourcePoint.X)) / l;
				double yd = (this.ArrowSize * (targetPoint.Y - sourcePoint.Y)) / l;
				targetPoint = new Point(sourcePoint.X + xd, sourcePoint.Y + yd);

				RotateTransform rt1 = new RotateTransform(this.ArrowAngle, sourcePoint.X, sourcePoint.Y);
				Point targetPoint1 = rt1.Transform(targetPoint);
				RotateTransform rt2 = new RotateTransform(-this.ArrowAngle, sourcePoint.X, sourcePoint.Y);
				Point targetPoint2 = rt2.Transform(targetPoint);

				GeneralTransform gt = source.TransformToVisual(this);
				targetPoint1 = gt.Transform(targetPoint1);
				targetPoint2 = gt.Transform(targetPoint2);
				sourcePoint = gt.Transform(sourcePoint);

				this.ln.Fill = source.Stroke;
				this.ln.Stroke = source.Stroke;
				this.ln.Points.Clear();

				this.ln.Points.Add(sourcePoint);
				this.ln.Points.Add(targetPoint1);
				this.ln.Points.Add(targetPoint2);

				double right = Math.Max(sourcePoint.X, Math.Max(targetPoint1.X, targetPoint2.X));
				double bottom = Math.Max(sourcePoint.Y, Math.Max(targetPoint1.Y, targetPoint2.Y));

				this.ln.Arrange(new Rect(0, 0, right, bottom));
			}

			return finalSize;
		}

		#region Override Methods - Visual Children

		protected override int VisualChildrenCount
		{
			get { return this.children.Count + ((this.Child == null) ? 0 : 1); }
		}

		protected override Visual GetVisualChild(int index)
		{
			if ((index == this.children.Count) && (this.Child != null))
				return this.Child;
			return this.children[index];
		}

		#endregion
		#region Properties

		private static readonly DependencyProperty ArrowSizeProperty = DependencyProperty.Register(
			"ArrowSize", typeof(double), typeof(ArrowDecorator));
		private static readonly DependencyProperty ArrowAngleProperty = DependencyProperty.Register(
			"ArrowAngle", typeof(double), typeof(ArrowDecorator));

		public double ArrowSize
		{
			get { return (double)this.GetValue(ArrowDecorator.ArrowSizeProperty); }
			set { this.SetValue(ArrowDecorator.ArrowSizeProperty, value); }
		}

		public double ArrowAngle
		{
			get { return (double)this.GetValue(ArrowDecorator.ArrowAngleProperty); }
			set { this.SetValue(ArrowDecorator.ArrowAngleProperty, value); }
		}

		#endregion
	}
}
