using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Documents;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows;
using Softix.Orchestration.Core.Components;
using Softix.Drawing.Diagram.Wpf.Resources;
using System.Windows.Data;
using System.Globalization;

namespace Softix.Drawing.Diagram.Wpf.Controls
{
	internal sealed class ComponentLinkSelectionAdorner : Adorner
	{
		#region Private Helper Class

		private sealed class SourceButtonStyleConverter : IValueConverter
		{
			public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
			{
				if ((targetType != typeof(Style)) || (false == (value is ComponentLinkSourceSide)))
					throw new Exception("The method or operation is not implemented.");

				ComponentLinkSourceSide sourceSide = (ComponentLinkSourceSide)value;
				if (sourceSide == ComponentLinkSourceSide.Left)
					return StyleRepository.LinkSourceMoveButtonRight;
				// else
				return StyleRepository.LinkSourceMoveButtonLeft;
			}

			public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
			{
				throw new Exception("The method or operation is not implemented.");
			}
		}

		#endregion

		private Button btnMoveSource;
		private Button btnDelete;

		private Button btnMoveTargetLeft;
		private Button btnMoveTargetTop;
		private Button btnMoveTargetBottom;
		private Button btnMoveTargetRight;

		private VisualCollection visualChildren = null;
		private ComponentLinkHost host = null;
		private UIElement editor;

		internal ComponentLinkSelectionAdorner(UIElement adornedElement, ComponentLinkHost host)
			: base(host)
		{
			this.editor = adornedElement;
			this.host = host;
			this.visualChildren = new VisualCollection(this);

			// move source button
			this.btnMoveSource = new Button();
			Binding btnMoveSourceStyle = new Binding("SourceSide");
			btnMoveSourceStyle.Source = this.host.ComponentLink;
			btnMoveSourceStyle.Mode = BindingMode.OneWay;
			btnMoveSourceStyle.Converter = new SourceButtonStyleConverter();
			BindingOperations.SetBinding(this.btnMoveSource, Control.StyleProperty, btnMoveSourceStyle);
			this.btnMoveSource.Click += new RoutedEventHandler(OnButtonMoveSourceClick);
			this.visualChildren.Add(this.btnMoveSource);

			// delete link button
			this.btnDelete = new Button();
			this.btnDelete.Style = StyleRepository.LinkSourceDeleteButton;
			this.btnDelete.Click += new RoutedEventHandler(OnDeleteLinkButtonClick);
			this.visualChildren.Add(this.btnDelete);

			// move target left button
			this.btnMoveTargetLeft = new Button();
			this.btnMoveTargetLeft.Style = StyleRepository.LinkTargetMoveButtonLeft;
			this.btnMoveTargetLeft.Click += new RoutedEventHandler(OnButtonMoveTargetLeftClick);
			this.visualChildren.Add(this.btnMoveTargetLeft);
			// move target top button
			this.btnMoveTargetTop = new Button();
			this.btnMoveTargetTop.Style = StyleRepository.LinkTargetMoveButtonTop;
			this.btnMoveTargetTop.Click += new RoutedEventHandler(OnButtonMoveTargetTopClick);
			this.visualChildren.Add(this.btnMoveTargetTop);
			// move target right button
			this.btnMoveTargetRight = new Button();
			this.btnMoveTargetRight.Style = StyleRepository.LinkTargetMoveButtonRight;
			this.btnMoveTargetRight.Click += new RoutedEventHandler(OnButtonMoveTargetRightClick);
			this.visualChildren.Add(this.btnMoveTargetRight);
			// move target bottom button
			this.btnMoveTargetBottom = new Button();
			this.btnMoveTargetBottom.Style = StyleRepository.LinkTargetMoveButtonBottom;
			this.btnMoveTargetBottom.Click += new RoutedEventHandler(OnButtonMoveTargetBottomClick);
			this.visualChildren.Add(this.btnMoveTargetBottom);
		}

		private void OnDeleteLinkButtonClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.ParentModel.RemoveComponentLink(this.host.ComponentLink);
		}

		private void OnButtonMoveTargetBottomClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.DestinationSide = ComponentLinkDestinationSide.Bottom;
			this.InvalidateArrange();
		}

		private void OnButtonMoveTargetRightClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.DestinationSide = ComponentLinkDestinationSide.Right;
			this.InvalidateArrange();
		}

		private void OnButtonMoveTargetTopClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.DestinationSide = ComponentLinkDestinationSide.Top;
			this.InvalidateArrange();
		}

		private void OnButtonMoveTargetLeftClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.DestinationSide = ComponentLinkDestinationSide.Left;
			this.InvalidateArrange();
		}

		private void OnButtonMoveSourceClick(object sender, RoutedEventArgs e)
		{
			this.host.ComponentLink.SourceSide = 
				(this.host.ComponentLink.SourceSide == ComponentLinkSourceSide.Left)
				? ComponentLinkSourceSide.Right
				: ComponentLinkSourceSide.Left;
			this.InvalidateArrange();
		}

		#region Methods - Overrides

		protected override Size ArrangeOverride(Size finalSize)
		{
			GeneralTransform transform = this.editor.TransformToVisual(this);

			Point source = this.host.SourcePoint;
			Point target = this.host.DestinationPoint;

			double width, height;

			// move source button
			width = this.btnMoveSource.Width;
			height = this.btnMoveSource.Height;
			Rect r = new Rect(
				source.X - width / 2,
				source.Y - height / 2,
				width, height);
			this.btnMoveSource.Arrange(transform.TransformBounds(r));

			// delete link button
			width = this.btnDelete.Width;
			height = this.btnDelete.Height;
			r = new Rect(
				(source.X + target.X) / 2 - width / 2,
				(source.Y + target.Y) / 2 - width / 2,
				width, height);
			this.btnDelete.Arrange(transform.TransformBounds(r));

			// move target left button
			width = this.btnMoveTargetLeft.Width;
			height = this.btnMoveTargetLeft.Height;
			r = new Rect(
				target.X - width / 2,
				target.Y - height / 2,
				width, height);
			Rect rLeft = r; rLeft.X -= 12;
			Rect rTop = r; rTop.Y -= 12;
			Rect rRight = r; rRight.X += 12;
			Rect rBottom = r; rBottom.Y += 12;
			this.btnMoveTargetLeft.Arrange(transform.TransformBounds(rLeft));
			this.btnMoveTargetTop.Arrange(transform.TransformBounds(rTop));
			this.btnMoveTargetRight.Arrange(transform.TransformBounds(rRight));
			this.btnMoveTargetBottom.Arrange(transform.TransformBounds(rBottom));

			// Return the final size.
			return finalSize;
		}

		protected override int VisualChildrenCount { get { return this.visualChildren.Count; } }
		protected override Visual GetVisualChild(int index) { return this.visualChildren[index]; }

		#endregion
	}
}
