using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Documents;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Input;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.IO;
using System.Xml;
using System.Resources;
using Softix.Drawing.Diagram.Wpf.Resources;

namespace Softix.Drawing.Diagram.Wpf.Controls
{
	internal sealed class ComponentSelectionAdorner : Adorner
	{
		private Thumb tmbTopLeft = null;
		private Thumb tmbBottomLeft = null;
		private Thumb tmbBottomRight = null;
		private Thumb tmbTopRight = null;
		private Thumb tmbCenter = null;

		private VisualCollection visualChildren = null;
		private ComponentHost host = null;

		public ComponentSelectionAdorner(ComponentHost host)
			: base(host)
		{
			this.host = host;
			this.visualChildren = new VisualCollection(this);

			this.tmbTopLeft = this.BuildAdornerThumb(Cursors.SizeNWSE, StyleRepository.TopLeftThumbStyle);
			this.tmbTopLeft.DragDelta += this.ResizeHeightReverse;
			this.tmbTopLeft.DragDelta += this.ResizeWidthReverse;
			this.tmbTopLeft.DragDelta += this.MoveLeft;
			this.tmbTopLeft.DragDelta += this.MoveTop;

			this.tmbBottomLeft = this.BuildAdornerThumb(Cursors.SizeNESW, StyleRepository.BottomLeftThumbStyle);
			this.tmbBottomLeft.DragDelta += this.ResizeWidthReverse;
			this.tmbBottomLeft.DragDelta += this.ResizeHeight;
			this.tmbBottomLeft.DragDelta += this.MoveLeft;

			this.tmbBottomRight = this.BuildAdornerThumb(Cursors.SizeNWSE, StyleRepository.BottomRightThumbStyle);
			this.tmbBottomRight.DragDelta += this.ResizeWidth;
			this.tmbBottomRight.DragDelta += this.ResizeHeight;

			this.tmbTopRight = this.BuildAdornerThumb(Cursors.SizeNESW, StyleRepository.TopRightThumbStyle);
			this.tmbTopRight.DragDelta += this.ResizeHeightReverse;
			this.tmbTopRight.DragDelta += this.ResizeWidth;
			this.tmbTopRight.DragDelta += this.MoveTop;

			this.tmbCenter = this.BuildAdornerThumb(Cursors.SizeAll, StyleRepository.CenterThumbStyle);
			this.tmbCenter.DragDelta += this.MoveLeft;
			this.tmbCenter.DragDelta += this.MoveTop;
		}

		#region Methods - Privates

		private Thumb BuildAdornerThumb(Cursor thumbCursor, Style style)
		{
			Thumb t = new Thumb();
			t.Width = t.Height = 10;
			t.Cursor = thumbCursor;
			t.Opacity = 0.3;
			t.Background = Brushes.MediumBlue;
			t.Style = style;

			this.visualChildren.Add(t);
			return t;
		}

		#endregion
		#region Methods - Overrides

		protected override Size ArrangeOverride(Size finalSize)
		{
			// desiredWidth and desiredHeight are the width and height of the element that's being adorned.  
			// These will be used to place the ResizingAdorner at the corners of the adorned element.  
			double desiredWidth = this.AdornedElement.DesiredSize.Width;
			double desiredHeight = this.AdornedElement.DesiredSize.Height;

			// adornerWidth & adornerHeight are used for placement as well.
			double adornerWidth = this.DesiredSize.Width;
			double adornerHeight = this.DesiredSize.Height;

			this.tmbTopLeft.Arrange(new Rect(
				-adornerWidth / 2 + 2,
				-adornerHeight / 2 + 2,
				adornerWidth, adornerHeight));
			this.tmbBottomLeft.Arrange(new Rect(
				-adornerWidth / 2 + 2,
				adornerHeight / 2 - 2,
				adornerWidth, adornerHeight));
			this.tmbBottomRight.Arrange(new Rect(
				adornerWidth / 2 - 2,
				adornerHeight / 2 - 2,
				adornerWidth, adornerHeight));
			this.tmbTopRight.Arrange(new Rect(
				adornerWidth / 2 - 2,
				-adornerHeight / 2 + 2,
				adornerWidth, adornerHeight));
			this.tmbCenter.Arrange(new Rect(
				0, 0, adornerWidth, adornerHeight));

			// Return the final size.
			return finalSize;
		}

		void EnforceSize(FrameworkElement adornedElement)
		{
			if (adornedElement.Width.Equals(Double.NaN))
				adornedElement.Width = adornedElement.DesiredSize.Width;
			if (adornedElement.Height.Equals(Double.NaN))
				adornedElement.Height = adornedElement.DesiredSize.Height;
		}

		protected override int VisualChildrenCount { get { return this.visualChildren.Count; } }
		protected override Visual GetVisualChild(int index) { return this.visualChildren[index]; }

		#endregion
		#region Methods - Event Handlers

		private void MoveLeft(object sender, DragDeltaEventArgs args)
		{
			double left = Canvas.GetLeft(this.host);
			left += args.HorizontalChange;
			Canvas.SetLeft(this.host, left);
		}

		private void MoveTop(object sender, DragDeltaEventArgs args)
		{
			double top = Canvas.GetTop(this.host);
			top += args.VerticalChange;
			Canvas.SetTop(this.host, top);
		}

		private void ResizeWidth(object sender, DragDeltaEventArgs args)
		{
			double width = this.host.Width;
			if (Double.IsNaN(width) == true)
				width = this.host.MinWidth;
			width += args.HorizontalChange;

			this.host.Width = Math.Max(width, this.host.MinWidth);
		}

		private void ResizeHeight(object sender, DragDeltaEventArgs args)
		{
			double height = this.host.Height;
			if (Double.IsNaN(height) == true)
				height = this.host.MinHeight;
			height += args.VerticalChange;
			this.host.Height = Math.Max(height, this.host.MinHeight);
		}

		private void ResizeWidthReverse(object sender, DragDeltaEventArgs args)
		{
			double width = this.host.Width;
			if (Double.IsNaN(width) == true)
				width = this.host.MinWidth;
			width -= args.HorizontalChange;
			this.host.Width = Math.Max(width, this.host.MinWidth);
		}

		private void ResizeHeightReverse(object sender, DragDeltaEventArgs args)
		{
			double height = this.host.Height;
			if (Double.IsNaN(height) == true)
				height = this.host.MinHeight;
			height -= args.VerticalChange;
			this.host.Height = Math.Max(height, this.host.MinHeight);
		}

		#endregion
	}
}
