using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using Softix.Drawing.Diagram.Wpf.Controls;

namespace Softix.Drawing.Diagram.Wpf.Core
{
	public sealed class ComponentPathInfo : DependencyObject
	{
		internal const string COMPONENT_PATH_NAME_MASK = "path_{0}";

		private ComponentPathHost host;

		public ComponentPathInfo()
		{
			this.PathId = Guid.NewGuid();
		}

		public ComponentPathInfo(string pathName, Guid pathId)
		{
			this.PathName = pathName;
			this.PathId = pathId;
		}

		#region Properties

		public static readonly DependencyProperty PathNameProperty = DependencyProperty.Register(
			"PathName", typeof(String), typeof(ComponentPathInfo));
		public static readonly DependencyProperty PathIdProperty = DependencyProperty.Register(
			"PathId", typeof(Guid), typeof(ComponentPathInfo));
		public static readonly DependencyProperty IsLinkedProperty = DependencyProperty.Register(
			"IsLinked", typeof(Boolean), typeof(ComponentPathInfo));

		public string PathName
		{
			get { return (string)this.GetValue(ComponentPathInfo.PathNameProperty); }
			set { this.SetValue(ComponentPathInfo.PathNameProperty, value); }
		}

		public Guid PathId
		{
			get { return (Guid)this.GetValue(ComponentPathInfo.PathIdProperty); }
			set { this.SetValue(ComponentPathInfo.PathIdProperty, value); }
		}

		public bool IsLinked
		{
			get { return (bool)this.GetValue(ComponentPathInfo.IsLinkedProperty); }
			set { this.SetValue(ComponentPathInfo.IsLinkedProperty, value); }
		}

		#endregion

		public string ControlName
		{
			get
			{
				return String.Format(
					COMPONENT_PATH_NAME_MASK,
					this.PathId.ToString().Replace("-", ""));
			}
		}
		internal ComponentPathHost Host
		{
			get { return this.host; }
			set
			{
				if (this.host != null)
					throw new InvalidOperationException();

				this.host = value;
				this.host.Name = this.ControlName;
			}
		}
	}
}
