using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media;
using Softix.Drawing.Diagram.Wpf.Controls;
using System.Windows.Documents;
using Softix.Drawing.Diagram.Wpf.Core;
using System.Windows.Input;
using System.Windows;
using Softix.Drawing.Diagram.Wpf.Resources;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;
using Softix.Orchestration.Core.Models.Events;
using System.Windows.Controls.Primitives;
using Softix.Orchestration.Core.Designers;
using System.Windows.Data;
using Softix.Orchestration.Core.Designers.HardCoded;
using System.Windows.Media.Effects;
using Softix.Orchestration.Core;

using DColor = System.Drawing.Color;

namespace Softix.Drawing.Diagram.Wpf
{
    public class ModelEditor : Canvas
    {
		private AdornerLayer adorners = null;
		private IAmModel model = null;
        private bool designMode = true;

		private ComponentHostList componentHosts;
		private Dictionary<IAmComponent, IDesignComponents> componentDesigners;
		private IDesignModels modelDesigner;
		private ComponentLinkHostList componentLinkHosts;

        public ModelEditor(IAmModel model)
        {
			// this.Background = Brushes.Snow;
			this.AllowDrop = true;
			this.Style = StyleRepository.DiagramEditorStyle;

			// create lists
			this.componentHosts = new ComponentHostList();
			this.componentDesigners = new Dictionary<IAmComponent, IDesignComponents>();
			this.componentLinkHosts = new ComponentLinkHostList();

			// model
			this.model = model;
			this.model.ComponentAdded += OnModelComponentAdded;
			this.model.ComponentRemoved += OnModelComponentRemoved;
			this.model.ComponentLinkAdded += OnModelComponentLinkAdded;
			this.model.ComponentLinkRemoved += OnModelComponentLinkRemoved;
			
			this.Loaded += new System.Windows.RoutedEventHandler(OnDiagramEditorLoaded);

			// thumb event handlers
			this.AddHandler(Thumb.DragStartedEvent, new DragStartedEventHandler(this.ThumbDragStarted));
			this.AddHandler(Thumb.DragDeltaEvent, new DragDeltaEventHandler(this.ThumbDragDelta));
			this.AddHandler(Thumb.DragCompletedEvent, new DragCompletedEventHandler(this.ThumbDragCompleted));
		}

		#region Methods - Private Model Event Handlers

		private void OnModelComponentAdded(IAmModel model, IAmComponent component)
		{
			ComponentHost host = new ComponentHost(this, component);
			this.componentHosts.Add(host);
			this.Children.Add(host);
		}

		private void OnModelComponentRemoved(IAmModel model, IAmComponent component)
		{
			if (this.componentHosts == null)
				return;

			ComponentHost host = this.componentHosts.FindByComponentId(component.ID);
			if ((this.componentSelector != null) && (this.componentSelector.AdornedElement == host))
			{
				this.adorners.Remove(this.componentSelector);
				this.componentSelector = null;
			}
			this.RemoveComponentDesigner(component);
			this.componentHosts.Remove(host);
			this.Children.Remove(host);
		}

		private void OnModelComponentLinkAdded(IAmModel model, IAmComponentLink link)
		{
			ComponentLinkHost host = new ComponentLinkHost(this, link);
			this.componentLinkHosts.Add(host);
			this.Children.Add(host);
		}

		private void OnModelComponentLinkRemoved(IAmModel model, IAmComponentLink link)
		{
			if (this.componentLinkHosts == null)
				return;

			ComponentLinkHost host = this.componentLinkHosts.FindByLinkId(link.ID);
			if ((this.componentSelector != null) && (this.componentSelector.AdornedElement == host))
			{
				this.adorners.Remove(this.componentSelector);
				this.componentSelector = null;
			}
			this.componentLinkHosts.Remove(host);
			this.Children.Remove(host);
		}

		#endregion
		#region Methods - Internals

		internal ComponentHost FindComponentHostById(Guid componentId)
		{
			return this.componentHosts.FindByComponentId(componentId);
		}

		internal ComponentLinkHost FindComponentLinkHostById(Guid linkId)
		{
			return this.componentLinkHosts.FindByLinkId(linkId);
		}

		internal DependencyObject FindHostAtPoint(Point point)
		{
			DependencyObject dobj = (this.InputHitTest(this.targetPoint) as DependencyObject);
			while ((dobj != null) &&
				(false == (dobj is ComponentHost)) &&
				(false == (dobj is ComponentLinkHost)))
			{
				dobj = VisualTreeHelper.GetParent(dobj);
			}
			return dobj;
		}

		private IDesignComponents GetComponentDesigner(IAmComponent component)
		{
			if (this.componentDesigners.ContainsKey(component) == true)
				return this.componentDesigners[component];

			IDesignComponents designer = ComponentDesignerFactory.CreateDesigner(component);
			if (designer == null)
				return null;
			this.componentDesigners[component] = designer;
			return designer;
		}

		private IDesignModels GetModelDesigner()
		{
			if ((this.modelDesigner != null) && (this.modelDesigner.Model == this.model))
				return this.modelDesigner;
			this.modelDesigner = new DefaultModelDesigner(this.model);
			return this.modelDesigner;
		}

		private IDesignStuff GetDesigner(DependencyObject dobj)
		{
			while ((dobj != null) &&
				(false == (dobj is ComponentHost)) &&
				(false == (dobj is ComponentLinkHost)))
			{
				dobj = VisualTreeHelper.GetParent(dobj);
			}

			if (dobj == null)
				return this.GetModelDesigner();
			else if (dobj is ComponentHost)
				return this.GetComponentDesigner(((ComponentHost)dobj).Component);
			else
				return null;
		}

		private void RemoveComponentDesigner(IAmComponent component)
		{
			if (this.componentDesigners.ContainsKey(component) == true)
				this.componentDesigners.Remove(component);
		}

		#endregion
		#region Methods - Adorners

		private Adorner componentSelector = null;

		protected override void OnMouseDown(MouseButtonEventArgs e)
		{
			base.OnMouseDown(e);
			if (this.adorners == null)
				return;

			DependencyObject dobj = (e.OriginalSource as DependencyObject);
			while ((dobj != null) &&
				(false == (dobj is ComponentHost)) &&
				(false == (dobj is ComponentLinkHost)))
			{
				dobj = VisualTreeHelper.GetParent(dobj);
			}

			if ((e.ChangedButton == MouseButton.Left) && (e.LeftButton == MouseButtonState.Pressed))
			{
				// change adorner
				if (this.componentSelector != null)
					this.adorners.Remove(this.componentSelector);
				if (dobj is ComponentHost)
				{
					this.adorners.Add(this.componentSelector = new ComponentSelectionAdorner(dobj as ComponentHost));
				}
				else if (dobj is ComponentLinkHost)
				{
					this.adorners.Add(this.componentSelector = new ComponentLinkSelectionAdorner(this, dobj as ComponentLinkHost));
				}

				if ((this.designMode == true) && (e.ClickCount == 2))
				{
					IDesignStuff designer = this.GetDesigner(dobj);
					if (designer != null)
						designer.ExecuteDefaultCommand();
				}
			}
			else if (
                (this.designMode == true) &&
                (e.ChangedButton == MouseButton.Right) &&
                (e.RightButton == MouseButtonState.Pressed) &&
                (dobj is ComponentHost)
                )
    		{
				IAmComponent component = ((ComponentHost)dobj).Component;
				IDesignComponents designer = this.GetComponentDesigner(component);
				if (designer != null)
				{
					ContextMenu menu = new ContextMenu();
					foreach (DesignerCommand cmd in designer.Commands)
					{
						if (String.IsNullOrEmpty(cmd.CommandLabel) == true)
						{
							menu.Items.Add(new Separator());
							continue;
						}
						MenuItem item = new MenuItem();
						item.DataContext = cmd;
						item.Header = cmd.CommandLabel;
						item.Click += new RoutedEventHandler(OnDesignerMenuItemClick);
						menu.Items.Add(item);
					}
					menu.IsOpen = true;
				}
			}
		}

		private void OnDesignerMenuItemClick(object sender, RoutedEventArgs e)
		{
			MenuItem item = (e.OriginalSource as MenuItem);
			if (item == null)
				return;

			DesignerCommand cmd = (item.DataContext as DesignerCommand);
			if (cmd != null)
				cmd.Select();
		}

		private void OnDiagramEditorLoaded(object sender, System.Windows.RoutedEventArgs e)
		{
			// get adorner layer
			this.adorners = AdornerLayer.GetAdornerLayer(this);

            // refresh model
            if (this.model == null)
                return;

            IAmComponent[] components = this.model.Components;
            if ((components == null) || (components.Length == 0))
                return;
            foreach (IAmComponent component in components)
            {
                this.OnModelComponentAdded(this.model, component);
            }

            IAmComponentLink[] componentLinks = this.model.ComponentLinks;
            if ((componentLinks == null) || (componentLinks.Length == 0))
                return;
            foreach (IAmComponentLink link in componentLinks)
            {
                this.OnModelComponentLinkAdded(this.model, link);
            }
		}

		#endregion

		public void ValidateModel()
		{
			IDesignModels designer = this.GetModelDesigner();
			if (designer != null)
				designer.Validate();
		}

		#region Drag Link Line

		private Line line = null;
		private Point targetPoint;
		private ComponentHost sourceHost;
		private IAmComponentOutboundPath sourcePath;
		private ComponentLinkSourceSide sourceSide;
		private ComponentHost targetHost;
		private ComponentLinkDestinationSide targetSide;

		private void ThumbDragStarted(object sender, DragStartedEventArgs args)
		{
			this.sourceHost = null;
			Thumb source = (args.OriginalSource as Thumb);
			if (source == null)
				return;

			DependencyObject dobj = source;
			while ((dobj != null) && (false == (dobj is ComponentHost)))
			{
				dobj = VisualTreeHelper.GetParent(dobj);
			}
			if (dobj == null)
				return;

			this.sourceHost = (ComponentHost)dobj;
			Guid pathId = ((ComponentPathInfo)((Thumb)args.OriginalSource).DataContext).PathId;
			this.sourcePath = Array.Find<IAmComponentOutboundPath>(this.sourceHost.Component.OutboundPaths,
				delegate(IAmComponentOutboundPath tmpPath)
				{
					return tmpPath.ID == pathId;
				});
			this.sourceSide = (source.Name == "LeftThumb") ? ComponentLinkSourceSide.Left : ComponentLinkSourceSide.Right;

			GeneralTransform t = source.TransformToVisual(this);
			Point sourcePoint = t.Transform(new Point(0, 0));
			this.line = this.CreateComponentLinkLine(sourcePoint);
			this.targetPoint = new Point(this.line.X2, this.line.Y2);
		}

		private void ThumbDragDelta(object sender, DragDeltaEventArgs args)
		{
			if ((this.line == null) || (this.sourceHost == null) || (this.sourcePath == null))
				return;

			this.targetPoint.X = this.line.X1 + args.HorizontalChange;
			this.targetPoint.Y = this.line.Y1 + args.VerticalChange;

			this.targetHost = (this.FindHostAtPoint(this.targetPoint) as ComponentHost);
			if (this.targetHost == null)
			{
				this.line.X2 = this.targetPoint.X;
				this.line.Y2 = this.targetPoint.Y;
				return;
			}
			Guid pathId = ((ComponentPathInfo)((Thumb)args.OriginalSource).DataContext).PathId;
			IAmComponentOutboundPath path = Array.Find<IAmComponentOutboundPath>(
				this.targetHost.Component.OutboundPaths,
				delegate(IAmComponentOutboundPath tmpPath)
				{
					return tmpPath.ID == pathId;
				});
			if (this.model.CanLinkComponents(this.sourceHost.Component, path, this.targetHost.Component) == false)
			{
				this.line.X2 = this.targetPoint.X;
				this.line.Y2 = this.targetPoint.Y;
				return;
			}

			#region get targeted side

			Rect rect = new Rect(
				(double)this.targetHost.GetValue(Canvas.LeftProperty),
				(double)this.targetHost.GetValue(Canvas.TopProperty),
				(double)this.targetHost.GetValue(Control.ActualWidthProperty),
				(double)this.targetHost.GetValue(Control.ActualHeightProperty));

			double yy = (rect.Width * (targetPoint.Y - rect.Top)) / rect.Height;
			bool aboveFirst = (targetPoint.X - rect.Left) > yy;
			bool aboveSecond = (rect.Width - (targetPoint.X - rect.Left)) > yy;

			double targetX = 0, targetY = 0;
			if ((aboveFirst == true) && (aboveSecond == true))
			{
				targetX = rect.Left + rect.Width / 2;
				targetY = rect.Top;
				this.targetSide = ComponentLinkDestinationSide.Top;
			}
			else if ((aboveFirst == false) && (aboveSecond == true))
			{
				targetX = rect.Left;
				targetY = rect.Top + rect.Height / 2;
				this.targetSide = ComponentLinkDestinationSide.Left;
			}
			else if ((aboveFirst == true) && (aboveSecond == false))
			{
				targetX = rect.Right;
				targetY = rect.Top + rect.Height / 2;
				this.targetSide = ComponentLinkDestinationSide.Right;
			}
			else if ((aboveFirst == false) && (aboveSecond == false))
			{
				targetX = rect.Left + rect.Width / 2;
				targetY = rect.Bottom;
				this.targetSide = ComponentLinkDestinationSide.Bottom;
			}

			#endregion
			this.line.X2 = targetX;
			this.line.Y2 = targetY;
		}

		private void ThumbDragCompleted(object sender, DragCompletedEventArgs args)
		{
			if (this.line != null)
			{
				this.RemoveComponentLinkLine(this.line);
				this.line = null;
			}

			if ((this.targetHost == null) ||
				(this.sourceHost == null) || (this.sourcePath == null))
				return;

			if (this.model.CanLinkComponents(this.sourceHost.Component, this.sourcePath, this.targetHost.Component) == true)
			{
				SimpleComponentLink link = new SimpleComponentLink();
				link.SetSourceComponent(this.sourceHost.Component, this.sourcePath.ID);
				link.SourceSide = this.sourceSide;
				link.TargetComponent = this.targetHost.Component;
				link.DestinationSide = this.targetSide;
				this.model.AddComponentLink(link);
			}
		}

		#region Methods - Internals

		internal Line CreateComponentLinkLine(Point location)
		{
			Line line = new Line();
			this.Children.Add(line);
			line.Stroke = Brushes.Green;
			line.X1 = line.X2 = location.X;
			line.Y1 = line.Y2 = location.Y;

			return line;
		}

		internal void RemoveComponentLinkLine(Line line)
		{
			this.Children.Remove(line);
		}

		#endregion

		#endregion

		#region Properties

		public IAmModel Model { get { return this.model; } }

		#endregion

        public bool DesignMode
        {
            get { return this.designMode; }
            set
            {
                this.designMode = value;

                if (this.designMode == true)
                {
                    #region remove bitmap effects

                    foreach (ComponentHost host in this.componentHosts)
                    {
                        host.BitmapEffect = null; // new OuterGlowBitmapEffect();
                    }
                    foreach (ComponentLinkHost host in this.componentLinkHosts)
                    {
                        host.BitmapEffect = null; // new OuterGlowBitmapEffect();
                    }

                    #endregion
                }
            }
        }

        public void HighlightComponent(IAmComponent component, DColor color)
        {
            ComponentHost host = this.componentHosts.Find(
                delegate(ComponentHost tmpHost)
                {
                    return (tmpHost.ComponentId == component.ID);
                });
            if (host == null)
                return;

            this.HighlightUIElement(host, color);
        }

        public void HighlightComponentLink(IAmComponentLink link, DColor color)
        {
            ComponentLinkHost host = this.componentLinkHosts.Find(
                delegate(ComponentLinkHost tmpHost)
                {
                    return (tmpHost.ComponentLink.ID == link.ID);
                });
            if (host == null)
                return;

            this.HighlightUIElement(host, color);
        }

        private void HighlightUIElement(UIElement element, DColor color)
        {
            OuterGlowBitmapEffect effect = new OuterGlowBitmapEffect();
            effect.GlowColor = Color.FromArgb(color.A, color.R, color.G, color.B);
            element.BitmapEffect = effect;
        }

		protected override void OnDrop(DragEventArgs e)
		{
			e.Handled = true;

			try
			{
				ComponentInfo cInfo = (e.Data.GetData(typeof(ComponentInfo)) as ComponentInfo);
				if (cInfo == null)
					return;
				IAmComponent component = Activator.CreateInstance(cInfo.Type) as IAmComponent;
				Point p = e.GetPosition(this);
				component.Left = p.X;
				component.Top = p.Y;
				this.model.AddComponent(component);
			}
			catch (Exception ex)
			{
				return;
			}
		}
	}
}
