using System;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Designers.HardCoded;
using System.IO;
using System.Reflection;

namespace Softix.Orchestration.Core.Designers
{
	public static class ComponentDesignerFactory
	{
		private static Dictionary<Type, Type> RegisteredDesigners;

		public static IDesignComponents CreateDesigner(IAmComponent component)
		{
			if (component == null)
				return null;

			// test designers
			if (RegisteredDesigners == null)
			{
				LoadDesigners();
			}

			Type componentType = component.GetType();
			while ((componentType != null) && (RegisteredDesigners.ContainsKey(componentType) == false))
			{
				componentType = componentType.BaseType;
			}
			if (componentType != null)
			{
				Type type = RegisteredDesigners[componentType];
				ConstructorInfo cInfo = type.GetConstructor(new Type[] { typeof(IAmComponent) });
				if (cInfo != null)
					return (IDesignComponents)cInfo.Invoke(new object[] { component });
			}

			return new DefaultComponentDesigner(component);
		}

		private static void LoadDesigners()
		{
			RegisteredDesigners = new Dictionary<Type,Type>();
			DirectoryInfo dir = new DirectoryInfo(Path.Combine(
				AppDomain.CurrentDomain.SetupInformation.ApplicationBase,
				"Designers"));
			if (dir.Exists == true)
			{
				foreach (FileInfo file in dir.GetFiles("*.dll"))
				{
					try
					{
						Assembly assembly = Assembly.LoadFile(file.FullName);
						if (assembly == null)
							continue;

						foreach (Type type in assembly.GetTypes())
						{
							if (typeof(IDesignComponents).IsAssignableFrom(type) == false)
								continue;

							object[] attrs = type.GetCustomAttributes(typeof(CanDesignComponentAttribute), true);
							if ((attrs != null) && (attrs.Length > 0))
							{
								foreach (Attribute attr in attrs)
								{
									CanDesignComponentAttribute cdc = (attr as CanDesignComponentAttribute);
									if (cdc == null)
										continue;

									Type componentType = cdc.ComponentType;
									if (componentType == null)
										continue;

									if (RegisteredDesigners.ContainsKey(componentType) == true)
										throw new InvalidOperationException(String.Format("Duplicate designer found for '{0}' components.", componentType.Name));
									RegisteredDesigners.Add(componentType, type);
								}
							}
						}
					}
					catch (InvalidOperationException) { throw; }
					catch { /* ignore */ }
				}
			}
		}
	}
}
