using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.Windows.Forms;
using System.Drawing;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Designers.Editors.MappingEditors;
using Softix.Orchestration.Core.Validation;

namespace Softix.Orchestration.Core.Designers.Editors
{
	public class ComponentInputPage : DesignerPage
	{
		protected IAmComponent component;
		private ListView lvwInputMappings = new ListView();
		private Panel panel;

		public ComponentInputPage(IAmComponent component)
		{
			this.component = component;
			this.Text = "Component Inputs";

			this.InitializeComponents();
			this.RefreshInputMappings();
		}

		#region Not Generated by Windows Forms Designer

		private void InitializeComponents()
		{
			this.Size = new Size(1024, 1024);

			SplitContainer sc = new SplitContainer();
			sc.FixedPanel = FixedPanel.Panel1;
			sc.Dock = DockStyle.Fill;
			this.Controls.Add(sc);
			sc.Panel1MinSize = sc.SplitterDistance = 275;

			this.lvwInputMappings = new ListView();
			this.lvwInputMappings.View = View.Details;
			this.lvwInputMappings.FullRowSelect = true;
			this.lvwInputMappings.HideSelection = false;
			this.lvwInputMappings.MultiSelect = false;
			this.lvwInputMappings.Columns.Add("Input Variable Name", 150);
			this.lvwInputMappings.Columns.Add("Data Type", 100);
			this.lvwInputMappings.Dock = DockStyle.Fill;
			this.lvwInputMappings.SelectedIndexChanged += this.OnInputMappingSelectedIndexChanged;
			sc.Panel1.Controls.Add(this.lvwInputMappings);
			this.panel = sc.Panel2;
		}

		#endregion

		#region DesignerPage Overrides

		public override System.Drawing.Image TabImage
		{
			get { return Resources.Resources.Import; }
		}

		#endregion

		private void RefreshInputMappings()
		{
			this.lvwInputMappings.Items.Clear();
			this.panel.Controls.Clear();

			if (this.component != null)
			{
				Variable[] vars = this.GetVariables();
				if ((vars != null) && (vars.Length != 0))
				{
					foreach (Variable var in vars)
					{
						ListViewItem item = new ListViewItem(new string[] { var.Name, var.DataType.DataTypeName });
						item.Tag = var;
						this.lvwInputMappings.Items.Add(item);
					}
				}
			}
		}

		protected virtual Variable[] GetVariables()
		{
			return InputVariableAttribute.GetInputDataDefinitions(this.component);
		}

		private void OnInputMappingSelectedIndexChanged(object sender, EventArgs e)
		{
			this.SavePanel2Controls();
			if ((this.lvwInputMappings.SelectedIndices != null) &&
				(this.lvwInputMappings.SelectedIndices.Count == 1))
			{
				ListViewItem item = this.lvwInputMappings.Items[this.lvwInputMappings.SelectedIndices[0]];
				if ((item != null) && (item.Tag is Variable))
				{
					VariableMappingEditor editor = null;
					if ((this.panel.Controls.Count == 1) &&
						(this.panel.Controls[0] is VariableMappingEditor))
					{
						editor = (VariableMappingEditor)this.panel.Controls[0];
						editor.Variable = (Variable)item.Tag;
					}

					if (editor == null)
					{
						editor = new VariableMappingEditor(
							 this.component,
							 (Variable)item.Tag);
						editor.Dock = DockStyle.Fill;
						this.panel.Controls.Clear();
						this.panel.Controls.Add(editor);
					}

					return;
				}
			}
			this.panel.Controls.Clear();
		}

		private void SavePanel2Controls()
		{
			if ((this.panel.Controls.Count == 1) &&
				(this.panel.Controls[0] is VariableMappingEditor))
			{
				((VariableMappingEditor)panel.Controls[0]).Save();
			}
		}

		public override void Close()
		{
			base.Close();

			this.SavePanel2Controls();
			if (this.component is ISupportValidation)
				((ISupportValidation)this.component).RunValidation();
		}
	}
}
