using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Softix.Orchestration.Core.Models;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data.Mappers;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.Core.Designers.Editors.MappingEditors
{
    public partial class ArrayVariableBuilder : UserControl, IVariableMapperUI
    {
        public ArrayVariableBuilder()
        {
            InitializeComponent();
        }

        #region IVariableMapperUI Members

        private IAmModel model = null;
        private IAmComponent component = null;
        private IAmArrayDataType target = null;

        public new void Load(IAmModel model, IAmComponent component, IAmDataType target, IVariableMapper mapper)
        {
            if ((target is IAmArrayDataType) == false)
                throw new InvalidOperationException("Target type must be an array.");

            this.model = model;
            this.component = component;
            this.target = (IAmArrayDataType)target;
            if (mapper is ArrayBuilderMapper)
            {
                ArrayBuilderMapper avm = (ArrayBuilderMapper)mapper;
                if (avm.Mappers != null)
                {
                    foreach (IVariableMapper m in avm.Mappers)
                    {
                        ListViewItem item = new ListViewItem(m.ToString());
                        item.Tag = m;
                        this.lvw.Items.Add(item);
                    }
                }
            }
        }

        public IVariableMapper Save()
        {
            ArrayBuilderMapper mapper = new ArrayBuilderMapper(this.target);

            foreach (ListViewItem item in this.lvw.Items)
            {
                if (item.Tag is IVariableMapper)
                    mapper.Mappers.Add((IVariableMapper)item.Tag);
            }

            return mapper;
        }

        #endregion

        private void btnAddSingle_Click(object sender, EventArgs e)
        {
            using(VariablePicker picker = new VariablePicker(this.component, target.ArrayItemDataType, null))
            {
                if (picker.ShowDialog(this) == DialogResult.OK)
                {
                    IVariableMapper mapper = picker.Mapper;
                    ListViewItem item = new ListViewItem(mapper.ToString());
                    item.Tag = mapper;
                    this.lvw.Items.Add(item);
                }
            }
        }

        private void btnAddArray_Click(object sender, EventArgs e)
        {
            using (VariablePicker picker = new VariablePicker(this.component, target, null))
            {
                if (picker.ShowDialog(this) == DialogResult.OK)
                {
                    IVariableMapper mapper = ArrayVariableMapper.Wrap(picker.Mapper);
                    ListViewItem item = new ListViewItem(mapper.ToString());
                    item.Tag = mapper;
                    this.lvw.Items.Add(item);
                }
            }
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if ((this.lvw.SelectedItems != null) && (this.lvw.SelectedItems.Count > 0))
            {
                foreach(ListViewItem item in this.lvw.SelectedItems)
                    this.lvw.Items.Remove(item);
            }
        }
    }
}
