using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Data.Mappers;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Core.Designers.Editors.MappingEditors
{
    public partial class VariableMappingEditor : UserControl
    {
        private IAmComponent component = null;
        private VariableMapping mapping = null;
        private TypeConverter converter = null;

        public VariableMappingEditor(IAmComponent component)
        {
            InitializeComponent();
            this.component = component;
        }

        public VariableMappingEditor(IAmComponent component, Variable var)
            : this(component)
        {
            this.Variable = var;
        }

        #region public Variable Variable { set; }

        private Variable variable;

        public Variable Variable
        {
            // get { return variable; }
            set
            {
                this.variable = value;
                this.gBox.Text = "Variable Mapping for '" + this.variable.Name + "'";
                this.lblType.Text = "Full Type Name: " + this.variable.DataType.DataType.FullName;
                this.lblTypeIsArray.Text = "Is Array: " + this.variable.IsArray;

                // set/create/add mapping
                VariableMapping mapping = null;
                if (this.component.InputDataMappings != null)
                {
                    mapping = this.component.InputDataMappings.Find(
                        delegate(VariableMapping tmpMapping)
                        {
                            return (tmpMapping.VariableName == this.variable.Name);
                        });
                }

                if (mapping == null)
                {
                    mapping = new VariableMapping(this.component, this.variable.Name);
                    this.component.InputDataMappings.Add(mapping);
                }

                this.mapping = mapping;

                // get converter
                this.converter = this.variable.DataType.DataTypeConverter;
                    // System.ComponentModel.TypeDescriptor.GetConverter(this.variable.DataType);
                if ((this.converter != null) && (converter.CanConvertFrom(typeof(String)) == false))
                    this.converter = null; // we don't care about it if it can't convert from string

                // set selected radio button
                if ((this.mapping.Mapper == null) ||
                    ((this.mapping.Mapper is ConstantValueMapper) && (converter == null)))
                    this.rbNone.Checked = true;
                else if (this.mapping.Mapper is ConstantValueMapper)
                {
                    Object val = ((ConstantValueMapper)this.mapping.Mapper).ConstantValue;
                    this.cmbConstant.Text = (val == null) ? "" : val.ToString();
                    this.rbConstant.Checked = true;
                }
                else
                {
                    this.rbVariable.Checked = true;
                }

                // set enabled
                if (converter == null)
                {
                    this.rbConstant.Enabled = false;
                    this.rbConstant.Checked = false;
                    this.cmbConstant.Enabled = false;
                    this.cmbConstant.Text = "no converter from string found";
                }
                this.rbConstant_CheckedChanged(this, EventArgs.Empty);
                this.rbVariable_CheckedChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        private void rbConstant_CheckedChanged(object sender, EventArgs e)
        {
            this.cmbConstant.Enabled = this.rbConstant.Checked;
        }

        private void rbVariable_CheckedChanged(object sender, EventArgs e)
        {
            if ((this.rbVariable.Checked == true) &&
                (this.panelVariable.Controls.Count == 0))
            {
                // create ui panel
                IVariableMapperUI ui = VariableMappingEditorFactory.GetMappingUIForType(
                    this.component.ParentModel,
                    this.component,
                    this.mapping.Mapper,
                    this.variable);
                if ((ui != null) && (ui is Control))
                {
                    Control uiControl = (Control)ui;
                    uiControl.Dock = DockStyle.Fill;
                    this.panelVariable.Controls.Clear();
                    this.panelVariable.Controls.Add(uiControl);
                }
            }

            this.panelVariable.Enabled = this.rbVariable.Checked;
        }

        public bool Save()
        {
            // get mapper
            IVariableMapper mapper = null;
            if (this.rbNone.Checked == true)
            {
                // it's ok... null mapper...
            }
            else if (this.rbConstant.Checked == true)
            {
                if (this.converter != null)
                {
                    try
                    {
                        Object val = this.converter.ConvertFrom(this.cmbConstant.Text);
                        mapper = new ConstantValueMapper(val);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(this, ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return false;
                    }
                }
            }
            else // we have a variable picker
            {
                if ((this.panelVariable.Controls.Count == 1) &&
                    (this.panelVariable.Controls[0] is IVariableMapperUI))
                {
                    mapper = ((IVariableMapperUI)this.panelVariable.Controls[0]).Save();
                }
            }

            this.mapping.Mapper = mapper;
            return true;
        }
    }
}