using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.Core.Designers.Editors
{
    public partial class VariableCollectionEditor : UserControl
    {
        public VariableCollectionEditor(IEnumerable<Variable> initialVariables)
        {
            InitializeComponent();

            if (initialVariables != null)
            {
                foreach (Variable var in initialVariables)
                {
                    this.lvw.Items.Add(new VariableListItem(var));
                }
            }
        }

        public IEnumerable<Variable> FinalVariables
        {
            get
            {
                foreach (ListViewItem item in this.lvw.Items)
                {
                    VariableListItem variableItem = (item as VariableListItem);
                    if (variableItem == null)
                        continue;

                    yield return variableItem.Variable;
                }
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Variable var = new Variable();
            var.RunValidation();
            this.lvw.Items.Add(new VariableListItem(var)).Selected = true;
        }

        private void lvw_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.lvw.SelectedItems.Count == 0)
            {
                this.pgrid.SelectedObject = null;
                return;
            }

            VariableListItem item = (this.lvw.SelectedItems[0] as VariableListItem);
            if (item == null)
            {
                this.pgrid.SelectedObject = null;
                return;
            }

            this.pgrid.SelectedObject = item.Variable;
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (this.lvw.SelectedItems.Count == 0)
                return;

            this.lvw.Items.Remove(this.lvw.SelectedItems[0]);
        }
    }
}
