using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Validation;
using System.Windows.Forms;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Designers.Editors;
using System.Windows;

using MessageBox = System.Windows.Forms.MessageBox;

namespace Softix.Orchestration.Core.Designers.HardCoded
{
	public class DefaultComponentDesigner : IDesignComponents
	{
		private IAmComponent component;

		public DefaultComponentDesigner(IAmComponent component)
		{
			this.component = component;
		}

		#region IDesignComponent Members

		public IAmComponent Component
		{
			get { return this.component; }
		}

		public virtual DesignerCommand[] Commands
		{
			get
			{
				List<DesignerCommand> list = new List<DesignerCommand>();

				DesignerCommand cmdEdit = new DesignerCommand("_Edit Component", "", this.OnEditComponentClick);
				list.Add(cmdEdit);
				DesignerCommand cmdDelete = new DesignerCommand("_Delete Component", "", this.OnDeleteComponentClick);
				list.Add(cmdDelete);
				DesignerCommand cmdValidate = new DesignerCommand("_Validate Component", "", this.OnForceValidateComponentClick);
				cmdValidate.Enabled = (this.component is ISupportValidation);
				list.Add(cmdValidate);
				DesignerCommand cmdSeparator = new DesignerCommand("");
				list.Add(cmdSeparator);
				DesignerCommand cmdBrowseData = new DesignerCommand("_Browse Data", "", this.OnBrowseComponentDataClick);
				list.Add(cmdBrowseData);

				return list.ToArray();
			}
		}

		public virtual void ShowPropertyPages()
		{
			EditingService.Edit(this, "Edit Component");
		}

		public virtual void Validate()
		{
			if (this.component is ISupportValidation)
			{
				ValidationBreak[] breaks = ((ISupportValidation)this.component).RunValidation();
				if ((breaks == null) || (breaks.Length == 0))
					MessageBox.Show("Component is valid.", "Validation", MessageBoxButtons.OK, MessageBoxIcon.Information);
				else
				{
					using (ValidationBreaksViewer viewer = new ValidationBreaksViewer(breaks))
					{
						viewer.ShowDialog();
					}
				}
			}
		}

		public virtual void ExecuteDefaultCommand()
		{
			this.ShowPropertyPages();
		}

		public virtual bool HasCustomStyle { get { return false; } }

		public virtual Style CustomStyle
		{
			get { throw new NotImplementedException(); }
		}

		#endregion
		#region IDesignStuff Members

		public DesignerPage[] Pages
		{
			get
			{
				List<DesignerPage> list = new List<DesignerPage>();
				list.Add(new PropertyGridPage(this.component));
				if (this.component is IAmEndComponent)
				{
					list.Add(new EndComponentInputPage((IAmEndComponent)this.component));
					Variable[] inputs = InputVariableAttribute.GetInputDataDefinitions(this.component);
					if ((inputs != null) && (inputs.Length > 0))
						list.Add(new ComponentInputPage(this.component));
				}
				else
					list.Add(new ComponentInputPage(this.component));
				return list.ToArray();
			}
		}

		#endregion

		#region Methods - DesignerCommand Event Handlers

		private void OnEditComponentClick(object sender, EventArgs e)
		{
			this.ShowPropertyPages();
		}

		private void OnDeleteComponentClick(object sender, EventArgs e)
		{
			this.component.ParentModel.RemoveComponent(this.component);
		}

		private void OnForceValidateComponentClick(object sender, EventArgs e)
		{
			this.Validate();
		}

		private void OnBrowseComponentDataClick(object sender, EventArgs e)
		{
			using (BrowseDataForm viewer = new BrowseDataForm(this.component))
			{
				viewer.ShowDialog();
			}
		}

		#endregion
	}
}
