using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Models;
using Softix.Orchestration.Core.Validation;
using System.Windows.Forms;
using Softix.Orchestration.Core.Designers.Editors;

namespace Softix.Orchestration.Core.Designers.HardCoded
{
	public class DefaultModelDesigner : IDesignModels
	{
		private IAmModel model;

		public DefaultModelDesigner(IAmModel model)
		{
			this.model = model;
		}

		#region IDesignModels Members

		public IAmModel Model
		{
			get { return this.model; }
		}

		public DesignerCommand[] Commands
		{
			get
			{
				List<DesignerCommand> list = new List<DesignerCommand>();

				DesignerCommand cmdEdit = new DesignerCommand("_Edit Model", "", this.OnEditModelClick);
				list.Add(cmdEdit);
				DesignerCommand cmdValidate = new DesignerCommand("_Validate Model", "", this.OnForceValidateModelClick);
				cmdValidate.Enabled = (this.model is ISupportValidation);
				list.Add(cmdValidate);

				return list.ToArray();
			}
		}

		public void ShowPropertyPages()
		{
            EditingService.Edit(this, "Edit Model");
        }

		public void Validate()
		{
			if (this.model is ISupportValidation)
			{
				ValidationBreak[] breaks = ((ISupportValidation)this.model).RunValidation();
				if ((breaks == null) || (breaks.Length == 0))
					MessageBox.Show("Model is valid.", "Validation", MessageBoxButtons.OK, MessageBoxIcon.Information);
				else
				{
					using (ValidationBreaksViewer viewer = new ValidationBreaksViewer(breaks))
					{
						viewer.ShowDialog();
					}
				}
			}
		}

		public void ExecuteDefaultCommand()
		{
            this.ShowPropertyPages();
		}

		#endregion
		#region IDesignStuff Members

		public DesignerPage[] Pages
		{
			get
            {
                return new DesignerPage[] 
                    {
                        new PropertyGridPage(this.model),
                        new ModelInputVariablesPage(this.model),
                        new ModelOutputVariablesPage(this.model)
                    };
            }
		}

		#endregion

        #region Methods - DesignerCommand Event Handlers

        private void OnEditModelClick(object sender, EventArgs args)
		{
            this.ShowPropertyPages();
		}

		private void OnForceValidateModelClick(object sender, EventArgs args)
		{
            this.Validate();
        }

        #endregion
    }
}
