using System;
using System.Collections.Generic;
using System.Text;
using NVelocity;
using NVelocity.App;
using Commons.Collections;
using System.IO;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Deployment.Internals
{
	internal static class GeneratorUtil
	{
		internal static string GenerateExecutorClass(
			IAmModel model,
			bool wcf,
			string @namespace,
			string className,
			params StartComponentInfo[] startComponents)
		{
			VelocityContext context = new VelocityContext();
			context.Put("model", model);
			context.Put("wcf", wcf);
			context.Put("namespace", @namespace);
			context.Put("className", className);
			context.Put("starts", startComponents);

			return GeneratorUtil.GenerateCode(context, "executor_class.nv");
		}

		internal static string GenerateModelReturnClass(IAmModel model, bool wcf, string @namespace)
		{
			VelocityContext context = new VelocityContext();
			context.Put("model", model);
			context.Put("wcf", wcf);
			context.Put("namespace", @namespace);

			return GeneratorUtil.GenerateCode(context, "model_return.nv");
		}

		private static string GenerateCode(VelocityContext context, string templateName)
		{
			string baseDir = Path.Combine(
				AppDomain.CurrentDomain.SetupInformation.ApplicationBase,
				"NVelocity");

			ExtendedProperties props = new ExtendedProperties();
			props.AddProperty("file.resource.loader.path", baseDir);
			Velocity.Init(props);

			Template template = Velocity.GetTemplate(templateName);
			using (StringWriter writer = new StringWriter())
			{
				template.Merge(context, writer);
				writer.Close();
				return writer.ToString();
			}
		}
	}
}
