using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Softix.Orchestration.Core;

namespace Softix.Orchestration.Editor.Controls
{
	public partial class ComponentsTree : UserControl
	{
		public ComponentsTree()
		{
			InitializeComponent();

			this.tvw.TreeViewNodeSorter = new ComponentTreeNodeComparer();
			this.RefreshNodes();
		}

		public void RefreshNodes()
		{
			this.tvw.Nodes.Clear();

			Dictionary<string, TreeNode> nodeDic = new Dictionary<string, TreeNode>();

			ComponentInfo[] infos = ComponentFactory.GetComponents();
			foreach (ComponentInfo info in infos)
			{
				string[] categoryPath = info.Category.Split('.');
				TreeNode categoryNode = this.FindCategoryNode(nodeDic, categoryPath);

				TreeNode newNode = new TreeNode(info.Name);
				newNode.ImageKey = newNode.SelectedImageKey = "cube";
				newNode.Tag = info;
				categoryNode.Nodes.Add(newNode);
			}

			this.tvw.Sort();
			this.tvw.ExpandAll();
		}

		private TreeNode FindCategoryNode(Dictionary<string, TreeNode> nodeDic, string[] categoryPath)
		{
			string path = String.Join("\\", categoryPath);
			if (nodeDic.ContainsKey(path) == true)
				return nodeDic[path];

			TreeNodeCollection parentNodes;

			if (categoryPath.Length == 1)
				parentNodes = this.tvw.Nodes;
			else
			{
				string[] newPath = new string[categoryPath.Length - 1];
				Array.Copy(categoryPath, newPath, newPath.Length);
				parentNodes = this.FindCategoryNode(nodeDic, newPath).Nodes;
			}

			TreeNode newNode = new TreeNode(categoryPath[categoryPath.Length - 1]);
			newNode.ImageKey = newNode.SelectedImageKey = "cubes";
			parentNodes.Add(newNode);
			nodeDic.Add(path, newNode);
			return newNode;
		}

		private void tvw_ItemDrag(object sender, ItemDragEventArgs e)
		{
			if (false == (e.Item is TreeNode))
				return;
			TreeNode node = (TreeNode)e.Item;
			if (false == (node.Tag is ComponentInfo))
				return;

			this.tvw.SelectedNode = node;
			this.tvw.DoDragDrop(node.Tag, DragDropEffects.Move);
		}
	}
}
