using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Editor.Execution
{
	internal sealed class DummyObject : ICustomTypeDescriptor
	{
		#region ICustomTypeDescriptor Members

		public AttributeCollection GetAttributes()
		{
			return new AttributeCollection();
		}

		public string GetClassName()
		{
			return "ModelInput";
		}

		public string GetComponentName()
		{
			return "ModelInput";
		}

		public TypeConverter GetConverter()
		{
			return null;
		}

		public EventDescriptor GetDefaultEvent()
		{
			return null;
		}

		public PropertyDescriptor GetDefaultProperty()
		{
			return null;
		}

		public object GetEditor(Type editorBaseType)
		{
			return null;
		}

		public EventDescriptorCollection GetEvents(Attribute[] attributes)
		{
			return null;
		}

		public EventDescriptorCollection GetEvents()
		{
			return null;
		}

		public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
		{
			return this.GetProperties();
		}

		public object GetPropertyOwner(PropertyDescriptor pd)
		{
			return this; // ???
		}

		#endregion

		private Variable[] variables;
		private Dictionary<string, object> variableValues;

		internal DummyObject(Variable[] variables)
		{
			this.variables = variables;
			if ((this.variables == null) || (this.variables.Length == 0))
				throw new InvalidOperationException("No variables specified.");
			foreach (Variable variable in this.variables)
			{
				variable.RunValidation();
				if (variable.IsValid == false)
					throw new InvalidOperationException("Variable '" + variable.Name + "' is not valid.");
			}

			this.variableValues = new Dictionary<string, object>();
		}

		public PropertyDescriptorCollection GetProperties()
		{
			List<PropertyDescriptor> list = new List<PropertyDescriptor>();

			foreach (Variable variable in this.variables)
			{
				list.Add(new DummyPropertyDescriptor(variable));
			}

			return new PropertyDescriptorCollection(list.ToArray(), true);
		}

		internal void ResetProperty(Variable variable)
		{
			this[variable] = variable.DataType.DefaultValue;
		}

		internal object this[Variable variable]
		{
			get
			{
				if (this.variableValues.ContainsKey(variable.Name) == true)
					return this.variableValues[variable.Name];
				return (this.variableValues[variable.Name] = variable.DataType.DefaultValue);
			}
			set { this.variableValues[variable.Name] = value; }
		}

		internal VariableValue[] GetVariableValues(Guid startComponentId)
		{
			List<VariableValue> list = new List<VariableValue>();

			foreach (Variable variable in this.variables)
				list.Add(new VariableValue(variable.Name, startComponentId, this[variable]));

			return list.ToArray();
		}
	}
}
