using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Xml;
using System.IO;
using System.IO.Compression;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters.Soap;

using Softix.Drawing.Diagram;
using Softix.Orchestration.Core;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;
using Softix.Drawing.Diagram.Wpf;
using Softix.Drawing.Diagram.Wpf.Core;
using Softix.Orchestration.Editor.Execution;
using Softix.Orchestration.Editor.Controls;
using Softix.Orchestration.Deployment;

namespace Softix.Orchestration.Editor
{
    public partial class MainForm : Form
    {
        private IAmModel model = null;
		private ModelEditorHost host = null;
		private ComponentsTree tree = null;

        public MainForm()
        {
            InitializeComponent();

			// create component tree
			this.tree = new ComponentsTree();
			this.tree.Dock = DockStyle.Fill;
			this.splitter.Panel1.Controls.Add(this.tree);

            // create model
            this.model = new SimpleModel();
		
			// setup WPF tab
			host = new ModelEditorHost(this.model);
			host.Dock = DockStyle.Fill;
			this.splitter.Panel2.Controls.Add(host);
		}

        private void toolBtnSave_Click(object sender, EventArgs e)
        {
            if (this.dlgSave.ShowDialog(this) == DialogResult.OK)
            {
                Persistance.FileStorage.SaveModel(this.model, this.dlgSave.FileName);
            }
        }

        private void toolBtnOpen_Click(object sender, EventArgs e)
        {
            if (this.dlgOpen.ShowDialog(this) == DialogResult.OK)
            {
                this.model = Persistance.FileStorage.LoadModel(this.dlgOpen.FileName);
                this.host.SetModel(this.model);
            }
        }

        private void toolBtnNew_Click(object sender, EventArgs e)
        {
			this.model = new SimpleModel();
			this.host.SetModel(this.model);
		}

        private void toolBtnRun_DropDownOpening(object sender, EventArgs e)
        {
            this.toolBtnRun.DropDownItems.Clear();

            IAmComponent[] starts = this.model.StartComponents;
            if ((starts != null) && (starts.Length > 0))
            {
                foreach (IAmComponent start in starts)
                {
                    ToolStripMenuItem item = new ToolStripMenuItem(start.Name); 
                    item.Tag = start;

                    this.toolBtnRun.DropDownItems.Add(item);
                }
            }
        }

        private void toolBtnRun_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
			IAmStartComponent start = (e.ClickedItem.Tag as IAmStartComponent);
            if (start != null)
            {
                ModelDebugger debugger = new ModelDebugger(this.host.Editor, start);
                this.splitter.Panel1.Controls.Clear();
                debugger.Dock = DockStyle.Fill;
                debugger.CloseDebugger +=
                    delegate
                    {
                        this.splitter.Panel1.Controls.Clear();
                        this.splitter.Panel1.Controls.Add(this.tree);
						this.tools.Enabled = true;
                    };
                this.splitter.Panel1.Controls.Add(debugger);
				this.tools.Enabled = false;
            }
        }

        private void toolBtnValidate_Click(object sender, EventArgs e)
        {
			this.host.Editor.ValidateModel();
		}

        private void toolBtnDeploy_DropDownOpening(object sender, EventArgs e)
        {
            this.toolBtnDeploy.DropDownItems.Clear();

            IDeployModels[] deployers = DeploymentFactory.GetAvailableDeploymentEngines();
            if ((deployers != null) && (deployers.Length > 0))
            {
                foreach (IDeployModels deployer in deployers)
                {
                    ToolStripMenuItem item = new ToolStripMenuItem(deployer.Label);
                    item.Enabled = deployer.CanDeploy(this.model);
                    item.Tag = deployer;

                    this.toolBtnDeploy.DropDownItems.Add(item);
                }
            }
        }

        private void toolBtnDeploy_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            IDeployModels deployer = (e.ClickedItem.Tag as IDeployModels);
            if (deployer != null)
            {
                deployer.Deploy(this.model);
            }
        }
    }
}