using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Components.HardCoded;

namespace Softix.Orchestration.Core
{
    public static class ComponentFactory
    {
        public static ComponentInfo[] GetComponents()
        {
            List<ComponentInfo> list = new List<ComponentInfo>();
            list.Add(ComponentAttribute.GetComponentInfo(typeof(StartExecutionComponent)));
			list.Add(ComponentAttribute.GetComponentInfo(typeof(EndExecutionComponent)));
			list.Add(ComponentAttribute.GetComponentInfo(typeof(ThrowExceptionComponent)));

            DirectoryInfo dir = new DirectoryInfo(Path.Combine(
                AppDomain.CurrentDomain.SetupInformation.ApplicationBase,
                "Components"));
            if (dir.Exists == true)
            {
                foreach (FileInfo file in dir.GetFiles("*.dll"))
                {
                    try
                    {
                        Assembly assembly = Assembly.LoadFile(file.FullName);
                        if (assembly == null)
                            continue;

                        foreach (Type type in assembly.GetTypes())
                        {
							if (type.IsAbstract == true)
								continue;

                            if (typeof(IAmComponent).IsAssignableFrom(type) == true)
                                list.Add(ComponentAttribute.GetComponentInfo(type));
                        }
                    }
                    catch { /* ignore */ }
                }
            }

            return list.ToArray();
        }
    }
}
