using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Components
{
    [AttributeUsage(AttributeTargets.Class, Inherited = false, AllowMultiple = false)]
    public sealed class ComponentAttribute : Attribute
    {
        private string name;
        private string category;
        private string description;

        public ComponentAttribute(string name, string category)
        {
            this.name = name;
            this.category = category;
        }

        public ComponentAttribute(string name, string category, string description)
            : this(name, category)
        {
            this.description = description;
        }

        public string Name { get { return this.name; } }
        public string Category { get { return this.category; } }
        public string Description { get { return this.description; } }

        internal static ComponentInfo GetComponentInfo(Type type)
        {
            ComponentInfo info = new ComponentInfo(type);

            object[] attrs = type.GetCustomAttributes(typeof(ComponentAttribute), false);
            if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is ComponentAttribute))
            {
                ComponentAttribute attr = (ComponentAttribute)attrs[0];
                info.Category = attr.Category;
                info.Name = attr.Name;
            }

            return info;
        }
    }
}
