using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Resources;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.Core.Components.HardCoded
{
    [ Serializable ]
    [ Component("End Execution", "Infrastructure") ]
    internal sealed class EndExecutionComponent : ComponentBase, IAmEndComponent
    {
        public EndExecutionComponent()
        {
            base.Name = "END";
        }

        public override ComponentOutput Run(DataStorage storage)
        {
			return new ComponentOutput();
		}

		public VariableValue[] EndExecution(DataStorage data)
		{
			List<VariableValue> list = new List<VariableValue>();
			VariableMappingList mappings = this.InputDataMappings;
			if ((mappings != null) && (mappings.Count > 0))
			{
				foreach (VariableMapping mapping in mappings)
				{
					Variable variable = Array.Find<Variable>(this.ParentModel.FinalVariables,
						delegate(Variable tmpVariable)
						{
							return (tmpVariable.Name == mapping.VariableName);
						});
					if (variable == null)
						continue;

					object value = mapping.Mapper.BuildData(data);
					if (value != null)
					{
						IAmDataType valueType = DataTypeFactory.GetDataType(value.GetType());
						IAmDataType variableType = variable.DataType;
						value = variableType.AssignFrom(valueType, value);
					}
					list.Add(new VariableValue(mapping.VariableName, this.ID, value));
				}
			}
			return list.ToArray();
		}

		#region design time only properties

		public override string ImageResourcePath
		{
			get { return ComponentImages.EndComponentImageResourcePath; }
		}

		#endregion

		#region IHasDynamicOutputData Members

		public Variable[] GetDynamicOutputDataDefinitions(string pathName)
		{
			if (this.ParentModel == null)
				return new Variable[0];

			Variable[] vars = this.ParentModel.FinalVariables;
			if ((vars == null) || (vars.Length == 0))
				return new Variable[0];

			List<Variable> list = new List<Variable>();
			foreach (Variable var in vars)
			{
				if (var.IsValid == true)
				{
					var.SourceComponentID = this.ID;
					list.Add(var);
				}
			}
			return list.ToArray();
		}

		#endregion
	}
}
