using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Resources;
using Softix.Orchestration.Core.Validation;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.Core.Components.HardCoded
{
    [Serializable]
    [Component("Start Execution", "Infrastructure")]
    internal sealed class StartExecutionComponent : ComponentBase, IHasDynamicOutputData, IAmStartComponent
    {
        private const string PATH_RUN = "run";

        public StartExecutionComponent()
        {
            base.Name = "START";
            base.AddOutboundPath(PATH_RUN);
        }

        public override ComponentOutput Run(DataStorage storage)
        {
			throw new InvalidOperationException("This method is not supported.");
		}

		public ComponentOutput StartExecution(VariableValue[] initialData)
		{
			ComponentOutput co = new ComponentOutput(base.GetPathID(StartExecutionComponent.PATH_RUN));
			if ((initialData != null) && (initialData.Length > 0))
			{
				foreach (VariableValue vv in initialData)
				{
					co[vv.Name] = vv.Value;
				}
			}
			return co;
		}

		#region design time only properties

		public override string ImageResourcePath
		{
			get { return ComponentImages.StartComponentImageResourcePath; }
		}

		#endregion

		protected override IEnumerable<ValidationBreak> GetValidationBreaks()
		{
			IEnumerable<ValidationBreak> breaks = base.GetValidationBreaks();
			if (breaks != null)
			{
				foreach(ValidationBreak @break in breaks)
					yield return @break;
			}

			if (this.ParentModel != null)
			{
				IAmComponentLink[] links = this.ParentModel.GetOutboundLinksForComponent(this);
				if ((links == null) || (links.Length == 0))
					yield return new ValidationBreak(this, "Start component must have at least one outbound link.", "");
			}
		}

		#region IHasDynamicOutputData Members

		public Variable[] GetDynamicOutputDataDefinitions(string pathName)
		{
			if (this.ParentModel == null)
				return new Variable[0];

			Variable[] vars = this.ParentModel.InitialVariables;
			if ((vars == null) || (vars.Length == 0))
				return new Variable[0];

			List<Variable> list = new List<Variable>();
			foreach (Variable var in vars)
			{
				if (var.IsValid == true)
				{
					var.SourceComponentID = this.ID;
					list.Add(var);
				}
			}
			return list.ToArray();
		}

		#endregion
	}
}
