using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Resources;
using System.ComponentModel;

namespace Softix.Orchestration.Core.Components.HardCoded
{
    [ Serializable ]
    [ Component("Throw Exception", "Infrastructure") ]
    internal sealed class ThrowExceptionComponent : ComponentBase, IAmEndComponent
    {
		public ThrowExceptionComponent()
        {
            base.Name = "THROW";
        }

        public override ComponentOutput Run(DataStorage storage)
        {
			return new ComponentOutput();
		}

		#region Properties - Inputs

		[NonSerialized]
		private String text = String.Empty;

		[InputVariable]
		[Browsable(false)]
		public String ErrorMessage
		{
			get { return this.text; }
			set { this.text = value; }
		}

		#endregion

		public VariableValue[] EndExecution(DataStorage data)
		{
			throw new Exception(this.text);
		}

		#region design time only properties

		public override string ImageResourcePath
		{
			get { return ComponentImages.ThrowExceptionComponentResourcePath; }
		}

		#endregion

		#region IHasDynamicOutputData Members

		public Variable[] GetDynamicOutputDataDefinitions(string pathName)
		{
			if (this.ParentModel == null)
				return new Variable[0];

			Variable[] vars = this.ParentModel.FinalVariables;
			if ((vars == null) || (vars.Length == 0))
				return new Variable[0];

			List<Variable> list = new List<Variable>();
			foreach (Variable var in vars)
			{
				if (var.IsValid == true)
				{
					var.SourceComponentID = this.ID;
					list.Add(var);
				}
			}
			return list.ToArray();
		}

		#endregion
	}
}
