using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Models;
using System.ComponentModel;

namespace Softix.Orchestration.Core.Components
{
    [Serializable]
    public class SimpleComponentLink : IAmComponentLink
    {
        public SimpleComponentLink()
        {
            this.id = Guid.NewGuid();
        }

        #region public Guid ID { get; set; }

        private Guid id = Guid.Empty;

        public Guid ID
        {
            get { return id; }
            protected set { id = value; }
        }

        #endregion

        #region public IAmModel ParentModel { get; set; }

        private IAmModel parentModel = null;

        public IAmModel ParentModel
        {
            get { return parentModel; }
            set { parentModel = value; }
        }

        #endregion

        #region source

        private IAmComponent sourceComponent = null;
        private IAmComponentOutboundPath sourceComponentPath = null;

        public IAmComponent SourceComponent
        {
            get { return this.sourceComponent; }
        }

        public IAmComponentOutboundPath SourceComponentPath
        {
            get { return this.sourceComponentPath; }
        }

        public void SetSourceComponent(IAmComponent component, Guid componentPathID)
        {
            if (this.parentModel != null)
            {
                // validate source component on model
                IAmComponent testComp = this.parentModel.GetComponent(component.ID);
                if (testComp == null)
                    throw new InvalidOperationException("Source component not found on model");
            }

            // validate outbound path
            IAmComponentOutboundPath testPath = Array.Find<IAmComponentOutboundPath>(
                component.OutboundPaths,
                delegate(IAmComponentOutboundPath tmpPath)
                {
                    return (tmpPath.ID == componentPathID);
                });
            if (testPath == null)
                throw new InvalidOperationException("Source path not found.");

            // everything looks ok
            this.sourceComponent = component;
            this.sourceComponentPath = testPath;
        }

        #endregion

        #region target

        private IAmComponent targetComponent;

        public IAmComponent TargetComponent
        {
            get { return targetComponent; }
            set
            {
                if (this.parentModel != null)
                {
                    // validate target component on model
                    IAmComponent testComp = this.parentModel.GetComponent(value.ID);
                    if (testComp == null)
                        throw new InvalidOperationException("Target component not found on model");
                }

                this.targetComponent = value;
            }
        }

        #endregion

		#region design time properties

		private ComponentLinkSourceSide sourceSide;
		private ComponentLinkDestinationSide destinationSide;

		public ComponentLinkSourceSide SourceSide
		{
			get { return this.sourceSide; }
			set
			{
				if (this.sourceSide != value)
				{
					this.sourceSide = value;
					this.OnPropertyChanged("SourceSide");
				}
			}
		}

		public ComponentLinkDestinationSide DestinationSide
		{
			get { return this.destinationSide; }
			set
			{
				if (this.destinationSide != value)
				{
					this.destinationSide = value;
					this.OnPropertyChanged("DestinationSide");
				}
			}
		}

		#endregion

		#region INotifyPropertyChanged Members

        [ field: NonSerialized ]
		public event PropertyChangedEventHandler PropertyChanged;

		private void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}

		#endregion
	}
}
