using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data
{
    [ Serializable ]
    public sealed class DataStorage
    {
        private Dictionary<Guid, List<VariableValue>> storage = null;
        private Dictionary<KeyValuePair<Guid, String>, Object> privateData = null;

        public DataStorage()
        {
            this.storage = new Dictionary<Guid, List<VariableValue>>();
            this.privateData = new Dictionary<KeyValuePair<Guid, string>, object>();
        }

        #region Public Variables

        public void AddVariable(Guid compId, String varName, object varValue)
        {
            List<VariableValue> list = null;

            if (this.storage.ContainsKey(compId) == true)
            {
                list = this.storage[compId];
            }

            if (list == null)
            {
                list = new List<VariableValue>();
                this.storage.Add(compId, list);
            }

            VariableValue vv = list.Find(
                delegate(VariableValue tmpVV)
                {
                    return (tmpVV.Name == varName);
                });
            if (vv == null)
                list.Add(new VariableValue(varName, compId,  varValue));
            else
                vv.Value = varValue;
        }

        public Object GetVariableValue(Variable var)
        {
            Guid compID = var.SourceComponentID;
            if (this.storage.ContainsKey(compID) == false)
                return null;

            List<VariableValue> list = this.storage[compID];
            VariableValue value = list.Find(
                delegate(VariableValue tmpValue)
                {
                    return (tmpValue.Name == var.Name);
                });

            return (value == null) ? null : value.Value;
        }

        public VariableValue[] GetOutputForComponent(Guid compID)
        {
            if (this.storage.ContainsKey(compID) == false)
                return new VariableValue[0];

            return this.storage[compID].ToArray();
        }

        public Guid[] GetComponentIDs()
        {
            List<Guid> list = new List<Guid>();
            list.AddRange(this.storage.Keys);
            return list.ToArray();
        }

        #endregion

        #region Private Variables

        public void AddPrivateData(Guid sourceComponentId, string variableName, object value)
        {
            KeyValuePair<Guid, String> key = new KeyValuePair<Guid, String>(sourceComponentId, variableName);
            this.privateData[key] = value;
        }

        public bool GetPrivateDataPresent<T>(Guid SourceComponentId, string variableName)
        {
            KeyValuePair<Guid, String> key = new KeyValuePair<Guid,String>(SourceComponentId, variableName);
            if (this.privateData.ContainsKey(key) == false)
                return false;
            object value = this.privateData[key];
            return (value is T);
        }

        public T GetPrivateData<T>(Guid sourceComponentId, string variableName)
        {
            KeyValuePair<Guid, String> key = new KeyValuePair<Guid, String>(sourceComponentId, variableName);
            if (this.privateData.ContainsKey(key) == true)
            {
                object value = this.privateData[key];
                if ((value != null) && (value is T))
                    return (T)value;
            }
            return default(T);
        }

        public void RemovePrivateData(Guid sourceComponentId, string variableName)
        {
            KeyValuePair<Guid, String> key = new KeyValuePair<Guid, String>(sourceComponentId, variableName);
            this.privateData.Remove(key);
        }

        #endregion
    }
}
