using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;

namespace Softix.Orchestration.Core.Data.DataTypes
{
	public class ArrayDataTypeBase<ElementType> : DataTypeBase, IAmArrayDataType
	{
		public ArrayDataTypeBase()
		{
		}

		#region IAmArrayDataType Members

		public IAmDataType ArrayItemDataType
		{
			get { return DataTypeFactory.GetDataType(typeof(ElementType)); }
		}

		public override Type DataType
		{
			get { return typeof(IEnumerable<ElementType>); }
		}

		public override object DefaultValue
		{
			get { return Array.CreateInstance(this.DataType, 0); }
		}

        public override bool IsAssignableFrom(IAmDataType dataType)
        {
            if (false == (dataType is IAmArrayDataType))
                return false;

            IAmDataType sourceType = ((IAmArrayDataType)dataType).ArrayItemDataType;
            IAmDataType targetType = this.ArrayItemDataType;
            return targetType.IsAssignableFrom(sourceType);
        }

        public override object AssignFrom(IAmDataType dataType, object value)
        {
            if ((false == (value is IEnumerable)) || (this.IsAssignableFrom(dataType) == false))
                throw new InvalidCastException();

            IAmDataType sourceType = ((IAmArrayDataType)dataType).ArrayItemDataType;
            List<ElementType> list = new List<ElementType>();
            foreach (object source in (IEnumerable)value)
            {
                list.Add((ElementType)this.ArrayItemDataType.AssignFrom(sourceType, source));
            }
            return list.ToArray();
        }
        
		#endregion

		protected override void Serialize(BinaryWriter writer, object value)
		{
			if (false == (value is IEnumerable<ElementType>))
				throw new InvalidCastException();

			foreach(ElementType element in (IEnumerable<ElementType>)value)
			{
				string buffer = this.ArrayItemDataType.ToBase64(element);
				writer.Write(buffer);
			}
		}

		protected override object Deserialize(BinaryReader reader)
		{
			IList<ElementType> list = new List<ElementType>();

			while (reader.BaseStream.Position < reader.BaseStream.Length)
			{
				string buffer = reader.ReadString();
				list.Add((ElementType)this.ArrayItemDataType.FromBase64(buffer));
			}

			return list;
		}

		public override string DataTypeName
		{
			get
			{
				return "Array of " + DataTypeFactory.GetDataType(typeof(ElementType)).DataTypeName;
			}
		}
	}
}
