using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.IO;

namespace Softix.Orchestration.Core.Data.DataTypes
{
	public abstract class DataTypeBase : IAmDataType
	{
		private Guid dataTypeId;
		private String dataTypeName;

		public DataTypeBase()
		{
			object[] attrs = this.GetType().GetCustomAttributes(typeof(DataTypeAttribute), false);
			if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is DataTypeAttribute))
			{
				DataTypeAttribute attr = (DataTypeAttribute)attrs[0];
				this.dataTypeId = attr.DataTypeId;
				this.dataTypeName = attr.DataTypeName;
			}
		}

		protected DataTypeBase(Guid id, string name)
		{
			this.dataTypeId = id;
			this.dataTypeName = name;
		}

		#region IAmDataType Members

		public Guid DataTypeId
		{
			get { return this.dataTypeId; }
		}

		public virtual string DataTypeName
		{
			get { return this.dataTypeName; }
		}

		public virtual TypeConverter DataTypeConverter
		{
			get { return TypeDescriptor.GetConverter(this.DataType); }
		}

		public abstract Type DataType { get; }
		public abstract object DefaultValue { get; }
        public abstract bool IsAssignableFrom(IAmDataType dataType);
        public abstract object AssignFrom(IAmDataType dataType, object value);

		#endregion

		#region Methods - Serialization

		protected abstract Object Deserialize(BinaryReader reader);
		protected abstract void Serialize(BinaryWriter writer, Object value);

		public object FromBase64(string serializedData)
		{
			byte[] buffer = Convert.FromBase64String(serializedData);
			using (MemoryStream stream = new MemoryStream(buffer))
			{
				return this.Deserialize(new BinaryReader(stream));
			}
		}

		public string ToBase64(object value)
		{
			using (MemoryStream stream = new MemoryStream())
			{
				this.Serialize(new BinaryWriter(stream), value);
				stream.Flush();
				stream.Close();
				return Convert.ToBase64String(stream.ToArray());
			}
		}

		#endregion
    }
}
