using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Softix.Orchestration.Core.Data.DataTypes
{
	public abstract class NumericDataTypeBase<WrappedType> : DataTypeBase, IAmNumericDataType
		where WrappedType : struct
	{
		public NumericDataTypeBase()
			: base()
		{ }

		public NumericDataTypeBase(Guid id, String name)
			: base(id, name)
		{ }

		#region IAmNumericDataType Members

		public virtual TypeCode TypeCode
		{
			get { return Type.GetTypeCode(typeof(WrappedType)); }
		}

		public override Type DataType
		{
			get { return typeof(WrappedType); }
		}

		public object ConvertTo(IAmNumericDataType type, object value)
		{
			return Convert.ChangeType(value, type.TypeCode);
		}

		public override object DefaultValue
		{
			get { return Convert.ChangeType(0, this.DataType); }
		}

        public override bool IsAssignableFrom(IAmDataType dataType)
        {
            return (dataType is IAmNumericDataType);
        }

        public override object AssignFrom(IAmDataType dataType, object value)
        {
            return Convert.ChangeType(value, this.DataType);
        }
        
		#endregion

        #region Methods - Serialization

        protected override object Deserialize(BinaryReader reader)
		{
			switch (this.TypeCode)
			{
				case TypeCode.Byte: return reader.ReadByte();
				case TypeCode.Decimal: return reader.ReadDecimal();
				case TypeCode.Double: return reader.ReadDouble();
				case TypeCode.Int16: return reader.ReadInt16();
				case TypeCode.Int32: return reader.ReadInt32();
				case TypeCode.Int64: return reader.ReadInt64();
				case TypeCode.SByte: return reader.ReadSByte();
				case TypeCode.Single: return reader.ReadSingle();
				case TypeCode.UInt16: return reader.ReadUInt16();
				case TypeCode.UInt32: return reader.ReadUInt32();
				case TypeCode.UInt64: return reader.ReadUInt64();
			}
			throw new NotSupportedException();
		}

		protected override void Serialize(BinaryWriter writer, object value)
		{
			switch (this.TypeCode)
			{
				case TypeCode.Byte: writer.Write((Byte)value); break;
				case TypeCode.Decimal: writer.Write((Decimal)value); break;
				case TypeCode.Double: writer.Write((Double)value); break;
				case TypeCode.Int16: writer.Write((Int16)value); break;
				case TypeCode.Int32: writer.Write((Int32)value); break;
				case TypeCode.Int64: writer.Write((Int64)value); break;
				case TypeCode.SByte: writer.Write((SByte)value); break;
				case TypeCode.Single: writer.Write((Single)value); break;
				case TypeCode.UInt16: writer.Write((UInt16)value); break;
				case TypeCode.UInt32: writer.Write((UInt32)value); break;
				case TypeCode.UInt64: writer.Write((UInt64)value); break;
			}
			throw new NotSupportedException();
        }

        #endregion
    }
}
