using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;

namespace Softix.Orchestration.Core.Data.DataTypes
{
    [DataType("{5719126b-3bdd-4484-b2b9-e1fb64dcf3ac}", "Text")]
	public sealed class StringDataType : DataTypeBase
	{
		public StringDataType()
			: base()
		{
		}

		public override Type DataType
		{
			get { return typeof(String); }
		}

		public override object DefaultValue
		{
			get { return String.Empty; }
		}

		protected override void Serialize(BinaryWriter writer, object value)
		{
			writer.Write((string)value);
		}

		protected override object Deserialize(BinaryReader reader)
		{
			return reader.ReadString();
		}

        public override bool IsAssignableFrom(IAmDataType dataType)
        {
            return true;
        }

        public override object AssignFrom(IAmDataType dataType, object value)
        {
            if ((dataType is IAmArrayDataType) && (value is IEnumerable))
            {
                List<String> list = new List<string>();
                foreach (object obj in (IEnumerable)value)
                {
                    list.Add((String)this.AssignFrom(((IAmArrayDataType)dataType).ArrayItemDataType, obj));
                }
                return String.Join(", ", list.ToArray());
            }
            else
            {
                return (value == null) ? String.Empty : value.ToString();
            }
        }
    }
}
