using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;

namespace Softix.Orchestration.Core.Data.Mappers
{
    [Serializable]
    public sealed class ArrayBuilderMapper : IVariableMapper, ISerializable, IArrayVariableMapper
    {
        public ArrayBuilderMapper(IAmDataType type)
        {
            this.mappers = new List<IVariableMapper>();
            this.type = (IAmArrayDataType)type;
        }

        #region public List<IVariableMapper> Mappers { get; set; }

        private List<IVariableMapper> mappers;

        public List<IVariableMapper> Mappers
        {
            get { return mappers; }
            set { mappers = value; }
        }

        IVariableMapper[] IArrayVariableMapper.ElementMappers
        {
            get { return (this.mappers == null) ? new IVariableMapper[0] : this.mappers.ToArray(); }
        }

        #endregion
        #region public IAmDataType Type { get; set; }

        private IAmArrayDataType type;

        public IAmArrayDataType Type
        {
            get { return type; }
            set { type = value; }
        }

        #endregion

		public Variable[] MappedVariables
		{
			get
			{
				List<Variable> list = new List<Variable>();
				if ((this.mappers != null) && (this.mappers.Count > 0))
				{
					foreach (IVariableMapper mapper in this.mappers)
					{
						Variable[] mappedVariables = mapper.MappedVariables;
						if ((mappedVariables != null) && (mappedVariables.Length > 0))
							list.AddRange(mapper.MappedVariables);
					}
				}
				return list.ToArray();
			}
		}

        public object BuildData(DataStorage storage)
        {
            IAmDataType type = this.type.ArrayItemDataType;

            Type genericType = typeof(List<>);
            Type listType = genericType.MakeGenericType(type.DataType);
            IList list = (IList)Activator.CreateInstance(listType);

            if (this.mappers != null)
            {
                foreach (IVariableMapper mapper in mappers)
                {
                    Object obj = mapper.BuildData(storage);
                    if (obj == null)
                        continue;
                    if (mapper is IArrayVariableMapper)
                    {
                        // array
                        IEnumerable enumerable = (IEnumerable)obj;
                        foreach (object thing in enumerable)
                        {
                            list.Add(thing);
                        }
                    }
                    else
                    {
                        // single variable
                        list.Add(obj);
                    }
                    //if (type.DataType.IsAssignableFrom(obj.GetType()) == true)
                    //{
                    //    list.Add(obj);
                    //}
                    //else if (typeof(IEnumerable).IsAssignableFrom(obj.GetType()) == true)
                    //{
                    //    foreach (object tmpData in (IEnumerable)obj)
                    //    {
                    //        list.Add(tmpData);
                    //    }
                    //}
                }
            }

            return list;
        }

		#region ISerializable Members

		public ArrayBuilderMapper(SerializationInfo info, StreamingContext context)
		{
			String typeName = info.GetString("_.typeName");
			this.type = (IAmArrayDataType)DataTypeFactory.GetDataType(System.Type.GetType(typeName));
			this.mappers = (List<IVariableMapper>)info.GetValue("_.mappers", typeof(List<IVariableMapper>));
		}

		public void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			info.AddValue("_.typeName", this.type.DataType.FullName);
			info.AddValue("_.mappers", this.mappers);
		}

		#endregion
	}
}
