using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Softix.Orchestration.Core.Data.Mappers
{
    [Serializable]
    public abstract class ArrayElementMapper : SingleVariableMapper
    {
        public ArrayElementMapper()
            : base()
        { }

        public ArrayElementMapper(Variable var)
            : base(var)
        { }

        #region IVariableMapper Members

        protected abstract Int32 GetIndex(DataStorage storage, IEnumerable array);

        protected override object GetData(DataStorage storage, object value)
        {
            IEnumerable array = value as IEnumerable;
            if (array == null)
                throw new InvalidOperationException("Variable '" + base.ToString() + "' is null or not an enumerable.");

            int index = this.GetIndex(storage, array);

            if (array is IList)
                return ((IList)array)[index];
            IEnumerator enumerator = array.GetEnumerator();
            enumerator.Reset();
            enumerator.MoveNext();
            for (int i = 0; i < index; i++)
            {
                enumerator.MoveNext();
            }
            return enumerator.Current;
        }

        #endregion
    }
}
