using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data.Mappers
{
    [Serializable]
    public sealed class ArrayVariableMapper : IArrayVariableMapper
    {
        private IVariableMapper mapper;

        private ArrayVariableMapper(IVariableMapper mapper)
        {
            this.mapper = mapper;
        }

        #region IArrayVariableMapper Members

        public IVariableMapper[] ElementMappers
        {
            get { return new IVariableMapper[] { this.mapper }; }
        }

        #endregion

        #region IVariableMapper Members

        public Variable[] MappedVariables
        {
            get { return (this.mapper == null) ? new Variable[0] : this.mapper.MappedVariables; }
        }

        public object BuildData(DataStorage storage)
        {
            return (this.mapper == null) ? null : this.mapper.BuildData(storage);
        }

        #endregion

        public static IVariableMapper Wrap(IVariableMapper mapper)
        {
            return new ArrayVariableMapper(mapper);
        }

        public override string ToString()
        {
            return "Array: " + ((this.mapper == null) ? "(no mapper)" : this.mapper.ToString());
        }
    }
}
