using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;

using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Core.Data
{
    [AttributeUsage(
        AttributeTargets.Property,
        Inherited = true,
        AllowMultiple = false
        )]
    public sealed class OutputVariableAttribute : Attribute
    {
        public OutputVariableAttribute(params string[] pathNames)
        {
            this.pathNames = pathNames;
        }

        #region public String[] PathNames { get; set; }

        private String[] pathNames;

        public String[] PathNames
        {
            get { return pathNames; }
            set { pathNames = value; }
        }

        #endregion
        #region public String Description { get; set; }

        private String description = String.Empty;

        public String Description
        {
            get { return description; }
            set { description = value; }
        }

        #endregion
        #region public Boolean IsLoopVariable { get; set; }

        private bool loopVariable = false;

        public bool IsLoopVariable
        {
            get { return this.loopVariable; }
            set { this.loopVariable = value; }
        }

        #endregion

        public static Variable[] GetOutputDataDefinitions(IAmComponent component, IAmComponentOutboundPath path)
        {
            if (component is IHasDynamicOutputData)
                return ((IHasDynamicOutputData)component).GetDynamicOutputDataDefinitions(path.Name);

            // look for attributes and build definition list
            VariableList list = new VariableList();

            foreach (PropertyInfo property in component.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                object[] attrs = property.GetCustomAttributes(typeof(OutputVariableAttribute), true);
                if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is OutputVariableAttribute))
                {
                    OutputVariableAttribute attr = (OutputVariableAttribute)attrs[0];
                    if (Array.Exists<String>(attr.PathNames,
                        delegate(String tmpPathName)
                        {
                            return (tmpPathName == path.Name);
                        }) == true)
                    {
                        Variable def = new Variable();
                        def.Name = property.Name;
                        def.DataType = DataTypeFactory.GetDataType(property.PropertyType);
                        def.SourceComponentID = component.ID;
                        def.IsLoopVariable = attr.IsLoopVariable;
                        list.Add(def);
                    }
                }
            }

            return list.ToArray();
        }

        public static void AddToComponentOutput(IAmComponent component, IAmComponentOutboundPath path, ComponentOutput co)
        {
            if (component is IHasDynamicOutputData)
                return; // we don't care about this component

            foreach (PropertyInfo property in component.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                object[] attrs = property.GetCustomAttributes(typeof(OutputVariableAttribute), true);
                if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is OutputVariableAttribute))
                {
                    OutputVariableAttribute attr = (OutputVariableAttribute)attrs[0];
                    if (Array.Exists<String>(attr.PathNames,
                        delegate(String tmpPathName)
                        {
                            return (tmpPathName == path.Name);
                        }) == true)
                    {

                        co[property.Name] = property.GetValue(component, null);
                    }
                }
            }
        }
    }
}
