using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Data.Mappers;
using Softix.Orchestration.Core.Validation;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Core.Data
{
    [Serializable]
    public sealed class VariableMapping : ValidationBase
    {
		private IAmComponent mappedComponent;

        public VariableMapping(IAmComponent mappedComponent)
        {
			this.mappedComponent = mappedComponent;
		}

        public VariableMapping(IAmComponent mappedComponent, String variableName)
			: this(mappedComponent)
        {
            this.variableName = variableName;
        }

        #region public String Variablename { get; set; }

        private String variableName;

        public String VariableName
        {
            get { return variableName; }
            set { variableName = value; }
        }

        #endregion
        #region public IVariableMapper Mapper { get; set; }

        private IVariableMapper mapper;

        public IVariableMapper Mapper
        {
            get { return mapper; }
            set { mapper = value; }
        }

        #endregion

        private IEnumerable<ValidationBreak> BaseValidationBreaks
        {
            get { return base.GetValidationBreaks(); }
        }

        protected override IEnumerable<ValidationBreak> GetValidationBreaks()
        {
            // parent reasons
            foreach (ValidationBreak vb in this.BaseValidationBreaks)
                yield return vb;

			if ((this.mappedComponent == null) || (this.mappedComponent.ParentModel == null))
			{
				yield return new ValidationBreak(this,
					"Invalid mapped component for variable '" + this.variableName + "'.",
					String.Empty,
					"Mapper");
			}

            if (this.mapper == null)
            {
                yield return new ValidationBreak(this,
                    "Mapper not set for variable '" + this.variableName + "'.",
                    "You must have a variable mapper for each input variable of a component.",
                    "Mapper");
            }
			else
			{
				Variable[] availableVars = this.mappedComponent.ParentModel.GetVariablesForComponent(this.mappedComponent.ID);
				Variable[] mappedVars = this.mapper.MappedVariables;
				List<Variable> notFoundVariables = new List<Variable>();
				if ((mappedVars != null) && (mappedVars.Length > 0))
				{
					notFoundVariables.AddRange(mappedVars);
					if ((availableVars != null) && (availableVars.Length > 0))
					{
						foreach (Variable variable in availableVars)
						{
							notFoundVariables.RemoveAll(
								delegate(Variable tmpVar)
								{
									return (String.Compare(tmpVar.Name, variable.Name, false) == 0);
								});
						}
					}
				}

				if (notFoundVariables.Count > 0)
				{
					foreach (Variable variable in notFoundVariables)
					{
						yield return new ValidationBreak(this,
							"Variable '" + variable.Name + "' not found in model.",
							String.Empty,
							"Mapper");
					}
				}
			}
        }

        public override string ToString()
        {
            return "Mapping: " + this.variableName;
        }
    }
}
