using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Globalization;

namespace Softix.Orchestration.Core.Data
{
    public sealed class VariableTypeConverter : TypeConverter
    {
        public VariableTypeConverter()
        { }

        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(String))
                return true;

            return base.CanConvertTo(context, destinationType);
        }

        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if ((value is IAmDataType) && (destinationType == typeof(String)))
            {
                return ((IAmDataType)value).DataTypeName;
            }

            return base.ConvertTo(context, culture, value, destinationType);
        }

        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(String))
            {
                return true;
            }
            return base.CanConvertFrom(context, sourceType);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            if (value is String)
            {
				string strValue = (String)value;
				IAmDataType[] types = DataTypeFactory.AllDataTypes;
				foreach (IAmDataType type in types)
				{
					if (type.DataTypeName == strValue)
						return type;
				}
                return null;
            }
            return base.ConvertFrom(context, culture, value);
        }

        public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
        {
            return true;
        }

        public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
        {
            return true;
        }

        public override StandardValuesCollection GetStandardValues(ITypeDescriptorContext context)
        {
            IAmDataType[] types = DataTypeFactory.AllDataTypes;
            StandardValuesCollection values = new StandardValuesCollection(types);
            return values;
        }

        public override bool IsValid(ITypeDescriptorContext context, object value)
        {
            if (value is IAmDataType)
            {
                return true;
            }
            return base.IsValid(context, value);
        }
    }
}
