using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Data.DataTypes;
using System.Collections;

namespace Softix.Orchestration.Core
{
	public static class DataTypeFactory
	{
        private static Dictionary<Type, IAmDataType> dataTypes;

		public static IAmDataType GetDataType(Type type)
		{
            DataTypeFactory.CheckDataTypesLoaded();

			if (type != null)
			{
				if (type.IsEnum == true)
				{
					Type generic = typeof(EnumDataType<>);
					Type result = generic.MakeGenericType(type);
					return (IAmDataType)Activator.CreateInstance(result);
				}

				if (type.IsArray == true)
				{
					Type generic = typeof(ArrayDataTypeBase<>);
					Type result = generic.MakeGenericType(type.GetElementType());
					return (IAmDataType)Activator.CreateInstance(result);
				}

				if (type.IsGenericType == true)
				{
					Type generic = typeof(IEnumerable<>);
					if (generic.IsAssignableFrom(type) == true)
					{
						Type[] args = type.GetGenericArguments();
					}
				}
			}

			bool isEnumerable = (typeof(IEnumerable).IsAssignableFrom(type) == true);

			while ((type != null) && (type != typeof(object)))
			{
				if (dataTypes.ContainsKey(type) == true)
					return dataTypes[type];
				type = type.BaseType;
			}

			if (isEnumerable == true)
				return new EnumerableDataType();
			return new ObjectDataType();

			//throw new InvalidOperationException(String.Format("Type handler for '{0}' type not found.", type.Name));
        }

        public static IAmDataType[] AllDataTypes
        {
            get
            {
                DataTypeFactory.CheckDataTypesLoaded();

                List<IAmDataType> list = new List<IAmDataType>();
                list.AddRange(dataTypes.Values);
                return list.ToArray();
            }
        }

        #region Methods - Private

        private static void CheckDataTypesLoaded()
        {
            if (dataTypes == null)
            {
                dataTypes = new Dictionary<Type, IAmDataType>();

                Type[] types = typeof(DataTypeFactory).Assembly.GetExportedTypes();
                FillTypeHandlerList(types);
            }
        }

        private static void FillTypeHandlerList(Type[] types)
        {
            foreach (Type iType in types)
            {
                if ((iType.IsAbstract == true) || (iType.IsInterface == true) || (iType.IsGenericTypeDefinition == true))
                    continue;

                Type[] interfaces = iType.FindInterfaces(
                    delegate(Type tmpType, Object criteria)
                    {
                        return (tmpType == (Type)criteria);
                    }, typeof(IAmDataType));
                if ((interfaces != null) && (interfaces.Length > 0))
                {
                    IAmDataType dataType = (IAmDataType)Activator.CreateInstance(iType);
                    dataTypes[dataType.DataType] = dataType;
                }
            }
        }

        #endregion
    }
}
