using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Models.Events;
using Softix.Orchestration.Core.Validation;

namespace Softix.Orchestration.Core.Models
{
	public interface IAmModel : ISupportValidation
    {
        String Name { get; set; }

        // add & remove components
        void AddComponent(IAmComponent component);
        void RemoveComponent(Guid componentID);
        void RemoveComponent(IAmComponent component);
        IAmComponent GetComponent(Guid componentID);
        IAmComponent[] Components { get; }

        event ModelComponentHandler ComponentAdded;
        event ModelComponentHandler ComponentRemoved;
        event ModelComponentHandler ComponentChanged;

        // add & remove component links
        void AddComponentLink(IAmComponentLink link);
        void RemoveComponentLink(Guid linkID);
        void RemoveComponentLink(IAmComponentLink link);
        IAmComponentLink GetComponentLink(Guid linkID);
        IAmComponentLink[] ComponentLinks { get; }
        IAmStartComponent[] StartComponents { get; }

        event ModelComponentLinkHandler ComponentLinkAdded;
        event ModelComponentLinkHandler ComponentLinkRemoved;
        event ModelComponentLinkHandler ComponentLinkChanged;

        IAmComponentLink[] GetOutboundLinksForComponent(Guid componentID);
        IAmComponentLink[] GetOutboundLinksForComponent(IAmComponent component);
        IAmComponentLink[] GetOutboundLinksForComponent(Guid componentID, Guid pathID);
        IAmComponentLink[] GetOutboundLinksForComponent(IAmComponent component, IAmComponentOutboundPath path);
        IAmComponentLink[] GetInboundLinksForComponent(Guid componentID);
        IAmComponentLink[] GetInboundLinksForComponent(IAmComponent component);

		bool CanLinkComponents(IAmComponent sourceComponent, IAmComponentOutboundPath sourceComponentPath, IAmComponent destinationComponent);

        // data definitions
        Variable[] InitialVariables { get; set; }
        Variable[] FinalVariables { get; set; }
        event ModelDataChanged InitialVariablesChanged;
        event ModelDataChanged FinalVariablesChanged;

        Variable[] GetVariablesForComponentLink(Guid linkID, Guid targetComponentId);
        Variable[] GetVariablesForComponentLink(IAmComponentLink link);
        Variable[] GetVariablesForComponent(Guid componentID);
        Variable[] GetVariablesForComponent(IAmComponent component);
    }
}
