using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using Microsoft.VisualBasic;
using System.ComponentModel;

namespace Softix.Orchestration.DefaultComponents.Financial
{
    [Serializable]
    [Component("FV - Future Value", "Financial")]
    public sealed class FVComponent : ComponentBase
    {
        private const string PATH_OUT = "FV";

        public FVComponent()
        {
            base.AddOutboundPath(PATH_OUT);
            base.Name = "Future Value";
        }

        #region Input: Rate

        [NonSerialized]
        private double rate;

        [ InputVariable ]
        [Browsable(false)]
        public double Rate
        {
            get { return this.rate; }
            set { this.rate = value; }
        }

        #endregion
        #region Input: PeriodCount

        [NonSerialized]
        private double periods;

        [InputVariable]
        [Browsable(false)]
        public double PeriodCount
        {
            get { return this.periods; }
            set { this.periods = value; }
        }

        #endregion
        #region Input: PeriodPayment

        [NonSerialized]
        private double payment;

        [InputVariable]
        [Browsable(false)]
        public double PeriodPayment
        {
            get { return this.payment; }
            set { this.payment = value; }
        }

        #endregion
        #region Input: CurrentValue

        [NonSerialized]
        private double current;

        [InputVariable]
        [Browsable(false)]
        public double CurrentValue
        {
            get { return this.current; }
            set { this.current = value; }
        }

        #endregion

        #region Output: FutureValue

        [NonSerialized]
        private double futureValue;

        [OutputVariable(PATH_OUT)]
        [Browsable(false)]
        public double FutureValue
        {
            get { return this.futureValue; }
            set { this.futureValue = value; }
        }

        #endregion

		#region Property: bool InvertValue

		private bool inverValue = true;

		[Category("2. Settings")]
		public bool InvertValue
		{
			get { return this.inverValue; }
			set { this.inverValue = value; }
		}

		#endregion

		public override ComponentOutput Run(DataStorage storage)
        {
            this.futureValue = Microsoft.VisualBasic.Financial.FV(
                this.rate,
                this.periods,
                this.payment,
                this.current,
                DueDate.EndOfPeriod);
			if (this.inverValue == true)
				this.futureValue = -this.futureValue;

            return new ComponentOutput(base.GetPathID(PATH_OUT));
        }
    }
}
