using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using System.ComponentModel;
using Microsoft.VisualBasic;

namespace Softix.Orchestration.DefaultComponents.Financial
{
    [Serializable]
    [Component("PMT - Payment", "Financial")]
    public sealed class PMTComponent : ComponentBase
    {
        private const string PATH_OUT = "PMT";

        public PMTComponent()
        {
            base.AddOutboundPath(PATH_OUT);
            base.Name = "Payment";
        }

        #region Input: Rate

        [NonSerialized]
        private double rate;

        [InputVariable]
        [Browsable(false)]
        public double Rate
        {
            get { return this.rate; }
            set { this.rate = value; }
        }

        #endregion
        #region Input: PeriodCount

        [NonSerialized]
        private double periods;

        [InputVariable]
        [Browsable(false)]
        public double PeriodCount
        {
            get { return this.periods; }
            set { this.periods = value; }
        }

        #endregion
        #region Input: PresentValue

        [NonSerialized]
        private double current;

        [InputVariable]
        [Browsable(false)]
        public double PresentValue
        {
            get { return this.current; }
            set { this.current = value; }
        }

        #endregion
        #region Input: FutureValue

        [NonSerialized]
        private double future;

        [InputVariable]
        [Browsable(false)]
        public double FutureValue
        {
            get { return this.future; }
            set { this.future = value; }
        }

        #endregion

        #region Output: PeriodPayment

        [NonSerialized]
        private double payment;

        [OutputVariable(PATH_OUT)]
        [Browsable(false)]
        public double PeriodPayment
        {
            get { return this.payment; }
            set { this.payment = value; }
        }

        #endregion

        public override ComponentOutput Run(DataStorage storage)
        {
            this.payment = Microsoft.VisualBasic.Financial.Pmt(
                this.rate,
                this.periods,
                this.current,
                this.future,
                DueDate.EndOfPeriod);

            return new ComponentOutput(base.GetPathID(PATH_OUT));
        }
    }
}
