using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.Drawing;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	public abstract class BaseImageProcessor : ComponentBase
    {
        private const string PATH = "Process";

        public BaseImageProcessor(string name)
        {
            base.AddOutboundPath(PATH);
            base.Name = name;
        }

        #region Input: OriginalImage

        [NonSerialized]
        private Image originalImage;

        [Browsable(false)]
        [InputVariable]
        public Image OriginalImage
        {
            get { return this.originalImage; }
            set { this.originalImage = value; }
        }

        #endregion
        #region Output: ProcessedImage

        [NonSerialized]
        private Image processedImage;

        [Browsable(false)]
        [OutputVariable(PATH)]
        public Image ProcessedImage
        {
            get { return this.processedImage; }
            set { this.processedImage = value; }
        }

        #endregion

        public override ComponentOutput Run(DataStorage storage)
        {
            this.processedImage = this.ProcessImage(this.originalImage);
            return new ComponentOutput(base.GetPathID(PATH));
        }

        protected abstract Image ProcessImage(Image original);
    }
}
