using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	public abstract class BasePerPixelFilter : BaseImageProcessor
    {
        public BasePerPixelFilter(string name)
            : base(name)
        {}

        protected override Image ProcessImage(Image original)
        {
			if (original == null)
				return null;

			// create new bitmap
			Bitmap newImage = new Bitmap(original.Width, original.Height, PixelFormat.Format32bppArgb);

			// copy original
			using (Graphics graphics = Graphics.FromImage(newImage))
			{
				graphics.DrawImage(original,
					new Rectangle(0, 0, newImage.Width, newImage.Height),
					new Rectangle(0, 0, original.Width, original.Height),
					GraphicsUnit.Pixel);
			}

			// transform image
			BitmapData bmpData = newImage.LockBits(
				new Rectangle(0, 0, newImage.Width, newImage.Height),
				ImageLockMode.ReadWrite,
				PixelFormat.Format32bppArgb);
			try
			{
				int stride = bmpData.Stride;
				int nOffset = stride - newImage.Width * 4;
				IntPtr scan0 = bmpData.Scan0;
				unsafe
				{
					byte* p = (byte*)(void*)scan0;
					for (int y = 0; y < newImage.Height; ++y)
					{
						for (int x = 0; x < newImage.Width; ++x)
						{
							Pixel pixel = this.ProcessPixel(new Pixel(p[3], p[2], p[1], p[0]));
							p[3] = pixel.Alpha;
							p[2] = pixel.Red;
							p[1] = pixel.Green;
							p[0] = pixel.Blue;
							p += 4;
						}
						p += nOffset;
					}
				}
			}
			finally
			{
				newImage.UnlockBits(bmpData);
			}

			return newImage;
        }

		protected abstract Pixel ProcessPixel(Pixel pixel);
    }
}
