using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	[Component("Adjust Brightness", "Imaging.Per Pixel Filters")]
	public sealed class BrightnessComponent : BasePerPixelFilter
	{
		public BrightnessComponent()
			: base("Brightness")
		{ }

		#region Input: Brightness

		[NonSerialized]
		private int brightness;

		[Browsable(false)]
		[InputVariable]
		public int Brightness
		{
			get { return this.brightness; }
			set { this.brightness = value; }
		}

		#endregion

		protected override Pixel ProcessPixel(Pixel pixel)
		{
			int red = (int)(pixel.Red) + this.brightness;
			int green = (int)(pixel.Green) + this.brightness;
			int blue = (int)(pixel.Blue) + this.brightness;

			return new Pixel(
				pixel.Alpha,
				(red < 0) ? (byte)0 : ((red > 255) ? (byte)255 : (byte)red),
				(green < 0) ? (byte)0 : ((green > 255) ? (byte)255 : (byte)green),
				(blue < 0) ? (byte)0 : ((blue > 255) ? (byte)255 : (byte)blue));
		}
	}
}
