using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	[Component("Color Filter", "Imaging.Per Pixel Filters")]
	public sealed class ColorFilterComponent : BasePerPixelFilter
	{
		public ColorFilterComponent()
			: base("Color Filter")
		{ }

		#region Input: AlphaOffset

		[NonSerialized]
		private int alphaOffset;

		[Browsable(false)]
		[InputVariable]
		public int AlphaOffset
		{
			get { return this.alphaOffset; }
			set { this.alphaOffset = value; }
		}

		#endregion
		#region Input: RedOffset

		[NonSerialized]
		private int redOffset;

		[Browsable(false)]
		[InputVariable]
		public int RedOffset
		{
			get { return this.redOffset; }
			set { this.redOffset = value; }
		}

		#endregion
		#region Input: GreenOffset

		[NonSerialized]
		private int greenOffset;

		[Browsable(false)]
		[InputVariable]
		public int GreenOffset
		{
			get { return this.greenOffset; }
			set { this.greenOffset = value; }
		}

		#endregion
		#region Input: BlueOffset

		[NonSerialized]
		private int blueOffset;

		[Browsable(false)]
		[InputVariable]
		public int BlueOffset
		{
			get { return this.blueOffset; }
			set { this.blueOffset = value; }
		}

		#endregion

		protected override Pixel ProcessPixel(Pixel pixel)
		{
			byte alpha = this.ProcessValue(pixel.Alpha, this.alphaOffset);
			byte red = this.ProcessValue(pixel.Red, this.redOffset);
			byte green = this.ProcessValue(pixel.Green, this.greenOffset);
			byte blue = this.ProcessValue(pixel.Blue, this.blueOffset);

			return new Pixel(alpha, red, green, blue);
		}

		private byte ProcessValue(byte value, int offset)
		{
			int iValue = value;
			iValue += offset;
			if (iValue < 0)
				iValue = 0;
			else if (iValue > 255)
				iValue = 255;
			return (byte)iValue;
		}
	}
}
