using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using System.ComponentModel;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	[Component("Adjust Contrast", "Imaging.Per Pixel Filters")]
	public sealed class ContrastComponent : BasePerPixelFilter
	{
		public ContrastComponent()
			: base("Contrast")
		{ }

		#region Input: Contrast

		[NonSerialized]
		private int contrast;

		[Browsable(false)]
		[InputVariable]
		public int Contrast
		{
			get { return this.contrast; }
			set { this.contrast = (value < -100) ? -100 : ((value > 100) ? 100 : value); }
		}

		#endregion

		protected override Pixel ProcessPixel(Pixel pixel)
		{
			double contrast = (100.0 + this.contrast) / 100.0;
			contrast *= contrast;

			return new Pixel(
				pixel.Alpha,
				this.ApplyContrast(pixel.Red, contrast),
				this.ApplyContrast(pixel.Green, contrast),
				this.ApplyContrast(pixel.Blue, contrast));
		}

		private byte ApplyContrast(byte value, double contrast)
		{
			//double pixel = value / 255.0;
			//pixel -= 0.5;
			//pixel *= contrast;
			//pixel += 0.5;
			//pixel *= 255;
			double pixel = (((value / 255.0) - 0.5) * contrast + 0.5) * 255.0;
			//if (pixel < 0) pixel = 0;
			//if (pixel > 255) pixel = 255;
			return (pixel < 0) ? (byte)0 : ((pixel > 255) ? (byte)255 : (byte)pixel);
		}
	}
}
