using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;
using System.Drawing;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
    [Serializable]
    [Component("Load Image from File", "Imaging")]
	public sealed class LoadImageComponent : ComponentBase
	{
		private const string PATH = "loaded";

		public LoadImageComponent()
		{
			base.AddOutboundPath(PATH);
			base.Name = "Load Image";
		}

		#region Input: FileName

		[NonSerialized]
		private string fileName;

		[Browsable(false)]
		[InputVariable]
		public string FileName
		{
			get { return this.fileName; }
			set { this.fileName = value; }
		}

		#endregion

		#region Output: LoadedImage

		[NonSerialized]
		private Image loadedImage;

		[Browsable(false)]
		[OutputVariable(PATH)]
		public Image LoadedImage
		{
			get { return this.loadedImage; }
			set { this.loadedImage = value; }
		}

		#endregion

		public override ComponentOutput Run(DataStorage storage)
		{
			this.loadedImage = Bitmap.FromFile(this.fileName);
			return new ComponentOutput(base.GetPathID(PATH));
		}
	}
}
