using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;
using System.Drawing;
using System.Drawing.Imaging;

namespace Softix.Orchestration.DefaultComponents.ImageProcessing
{
	[Serializable]
	[Component("Resize Image", "Imaging")]
	public sealed class ResizeImageComponent : BaseImageProcessor
	{
		public ResizeImageComponent()
			: base("Resize Image")
		{ }

		#region Input: Width

		[NonSerialized]
		private int width;

		[Browsable(false)]
		[InputVariable]
		public int TargetWidth
		{
			get { return this.width; }
			set { this.width = value; }
		}

		#endregion
		#region Input: Height

		[NonSerialized]
		private int height;

		[Browsable(false)]
		[InputVariable]
		public int TargetHeight
		{
			get { return this.height; }
			set { this.height = value; }
		}

		#endregion
		#region Input: BackgroundColor

		[NonSerialized]
		private string color;

		[Browsable(false)]
		[InputVariable]
		public string BackgroundColor
		{
			get { return this.color; }
			set { this.color = value; }
		}

		#endregion

		protected override Image ProcessImage(Image original)
		{
			Bitmap newImage = new Bitmap(this.width, this.height, PixelFormat.Format32bppArgb);

			int width, height;
			double a = ((double)original.Width) / ((double)original.Height);
			double b = ((double)newImage.Width) / ((double)newImage.Height);
			if (a > b)
			{
				width = newImage.Width;
				height = (original.Height * newImage.Width) / original.Width;
			}
			else
			{
				width = (original.Width * newImage.Height) / original.Height;
				height = newImage.Height;
			}

			using (Graphics g = Graphics.FromImage(newImage))
			{
				g.Clear(ColorTranslator.FromHtml(this.color));
				g.DrawImage(original,
					new Rectangle((newImage.Width - width) / 2, (newImage.Height - height) / 2, width, height),
					new Rectangle(0, 0, original.Width, original.Height),
					GraphicsUnit.Pixel);
			}

			return newImage;
		}
	}
}
