using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using System.ComponentModel;
using Softix.Orchestration.Core.Validation;
using Softix.Orchestration.Core;

namespace Softix.Orchestration.DefaultComponents.InsertData
{
	[Serializable]
	[Component("Insert Array", "Insert Data")]
	public sealed class InsertArray : ComponentBase, IHasDynamicOutputData
	{
		private const string PATH = "Array";
		private const string VARIABLE = "Array";

		public InsertArray()
		{
			base.AddOutboundPath(PATH);
			base.Name = "Insert Array";
		}

		#region public IAmDataType DataType

		[NonSerialized]
		private IAmDataType dataType = null;
		private string dataTypeName = null;

		[TypeConverter(typeof(VariableTypeConverter))]
		[Category("2. Settings")]
		public IAmDataType DataType
		{
			get
			{
				if ((this.dataType == null) && (String.IsNullOrEmpty(this.dataTypeName) == false))
					return (this.dataType = DataTypeFactory.GetDataType(Type.GetType(this.dataTypeName)));
				return this.dataType;
			}
			set
			{
				this.dataType = value;
				this.dataTypeName = (value == null) ? null : value.DataType.AssemblyQualifiedName;
				this.OnComponentChanged();
			}
		}

		#endregion

		#region Validation

		protected override IEnumerable<ValidationBreak> GetValidationBreaks()
		{
			IEnumerable<ValidationBreak> baseBreaks = base.GetValidationBreaks();
			if (baseBreaks != null)
			{
				foreach (ValidationBreak vbreak in baseBreaks)
					yield return vbreak;
			}

			if (this.DataType == null)
			{
				yield return new ValidationBreak(this, "You must specify a data type.", "You must specify a data type for the array items.", "DataType");
			}
		}

		#endregion

		#region IHasDynamicOutputData Members

		public Variable[] GetDynamicOutputDataDefinitions(string pathName)
		{
			this.RunValidation();
			if (this.IsValid == false)
				return new Variable[0];

			//Type genericType = typeof(List<>);
			//Type listType = genericType.MakeGenericType(this.dataType.DataType);
			Type listType = this.DataType.DataType.MakeArrayType();
			return new Variable[]
				{
					new Variable(this.ID, VARIABLE, DataTypeFactory.GetDataType(listType))
				};
		}

		#endregion

		public override ComponentOutput Run(DataStorage storage)
		{
			Type genericType = typeof(List<>);
			Type listType = genericType.MakeGenericType(this.dataType.DataType);

			ComponentOutput co = new ComponentOutput(base.GetPathID(PATH));
			co[VARIABLE] = Activator.CreateInstance(listType);
			return co;
		}
	}
}
