using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.Collections;
using Softix.Orchestration.Core.Data;
using System.ComponentModel;
using Softix.Orchestration.Core;

namespace Softix.Orchestration.DefaultComponents.Iterators
{
    [Serializable]
    [Component("ForEach", "Iterators")]
    public sealed class ForEach : ComponentBase, IHasDynamicOutputData
    {
        private const string PATH_END = "End Loop";
        private const string PATH_NEXT = "Next Item";

        public ForEach()
        {
            base.AddOutboundPath(PATH_END);
            base.AddOutboundPath(PATH_NEXT);

            base.Name = "For Each";
        }

        #region Properties

        [NonSerialized]
        private IEnumerable<Object> enumerable;

        [InputVariable]
        [Browsable(false)]
        public IEnumerable<Object> Enumerable
        {
            get { return this.enumerable; }
            set { this.enumerable = value; }
        }

        #endregion

        #region IHasDynamicOutputData Members

        public Variable[] GetDynamicOutputDataDefinitions(string pathName)
        {
            List<Variable> list = new List<Variable>();

            if (pathName == PATH_NEXT)
            {
                VariableMapping mapping = this.InputDataMappings.GetMappingForVariable("Enumerable");
                if ((mapping != null) && (mapping.Mapper != null))
                {
                    Variable[] variables = mapping.Mapper.MappedVariables;
                    if ((variables != null) && (variables.Length == 1) && (variables[0].DataType != null))
                    {
                        Type dataType = variables[0].DataType.DataType;
                        if ((dataType.IsGenericType) && (dataType.GetGenericTypeDefinition() == typeof(IEnumerable<>)))
                        {
                            Variable var = new Variable(
                                this.ID,
                                "LoopVariable",
                                DataTypeFactory.GetDataType(dataType.GetGenericArguments()[0]));
                            var.IsLoopVariable = true;
                            list.Add(var);
                        }
                        else
                        {
                            Variable var = new Variable(
                                this.ID,
                                "LoopVariable",
                                DataTypeFactory.GetDataType(typeof(Object)));
                            var.IsLoopVariable = true;
                            list.Add(var);
                        }
                    }
                }
            }

            return list.ToArray();
        }

        #endregion

        public override ComponentOutput Run(DataStorage storage)
        {
            IEnumerator enumerator = null;
            if (storage.GetPrivateDataPresent<IEnumerator>(this.ID, "enumerator") == false)
            {
                enumerator = this.enumerable.GetEnumerator();
                enumerator.Reset();
                storage.AddPrivateData(this.ID, "enumerator", enumerator);
            }
            else
                enumerator = storage.GetPrivateData<IEnumerator>(this.ID, "enumerator");

            if (enumerator.MoveNext() == false)
            {
                storage.RemovePrivateData(this.ID, "enumerator");
                return new ComponentOutput(base.GetPathID(PATH_END));
            }
            ComponentOutput output = new ComponentOutput(base.GetPathID(PATH_NEXT));
            output["LoopVariable"] = enumerator.Current;
            return output;
        }
    }
}
