using System;
using System.Collections.Generic;
using System.Text;
using Softix.Orchestration.Core.Components;
using System.ComponentModel;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Validation;

namespace Softix.Orchestration.DefaultComponents.Math
{
	[Serializable]
	[Component("Floating point operation", "Math")]
	public sealed class DoubleOperationComponent : ComponentBase
	{
		private const string PATH = "Compute";

		public DoubleOperationComponent()
		{
			base.AddOutboundPath(PATH);
			base.Name = "Compute";
		}

		#region Property: Operation

		private DoubleOperation operation = DoubleOperation.Unknown;

		[Category("2. Settings")]
		public DoubleOperation Operation
		{
			get { return this.operation; }
			set
			{
				this.operation = value;
				base.OnComponentChanged();
			}
		}

		#endregion

		#region Input: FirstNumber

		[NonSerialized]
		private double x;

		[Browsable(false)]
		[InputVariable]
		public double FirstNumber
		{
			get { return this.x; }
			set { this.x = value; }
		}

		#endregion
		#region Input: SecondNumber

		[NonSerialized]
		private double y;

		[Browsable(false)]
		[InputVariable]
		public double SecondNumber
		{
			get { return this.y; }
			set { this.y = value; }
		}

		#endregion

		#region Output: Result

		[NonSerialized]
		private double result;

		[Browsable(false)]
		[OutputVariable(PATH)]
		public double Result
		{
			get { return this.result; }
			set { this.result = value; }
		}

		#endregion

		#region Validation

		protected override IEnumerable<ValidationBreak> GetValidationBreaks()
		{
			IEnumerable<ValidationBreak> baseReasons = base.GetValidationBreaks();
			if (baseReasons != null)
			{
				foreach (ValidationBreak vb in baseReasons)
					yield return vb;
			}

			if (this.operation == DoubleOperation.Unknown)
				yield return new ValidationBreak(this, "Unknown mathematical operation", "", "Operation");
		}

		#endregion

		public override ComponentOutput Run(DataStorage storage)
		{
			switch (this.operation)
			{
				case DoubleOperation.Add:
					this.result = this.x + this.y;
					break;
				case DoubleOperation.Substract:
					this.result = this.x - this.y;
					break;
				case DoubleOperation.Multiply:
					this.result = this.x * this.y;
					break;
				case DoubleOperation.Divide:
					this.result = this.x / this.y;
					break;
				case DoubleOperation.Power:
					this.result = System.Math.Pow(this.x, this.y);
					break;
				default:
					this.result = Double.NaN;
					break;
			}
			return new ComponentOutput(base.GetPathID(PATH));
		}
	}

	public enum DoubleOperation
	{
		Unknown = 0,

		Add = 1,
		Substract = 2,
		Multiply = 3,
		Divide = 4,
		Power = 5
	}
}
