using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.DefaultComponents.Math
{
    [Serializable]
    [Component("Compare 2 Numbers", "Rules")]
    public sealed class CompareNumbersRule : ComponentBase
    {
        public const string PATH_SMALLER = "First number smaller";
        public const string PATH_SMALLER_OR_EQUAL = "First number smaller or equal";
        public const string PATH_EQUAL = "Numbers are equal";
        public const string PATH_BIGGER = "First number bigger";
        public const string PATH_BIGGER_OR_EQUAL = "First number bigger or equal";

        public CompareNumbersRule()
        {
            this.HowToTreatEqual = TreatEqualAsEnum.SeparatePath;
            base.Name = "Compare Numbers";
        }

        #region public int FirstNumber { get; set; }

        [NonSerialized]
        private int firstNumber = 0;

        [InputVariable]
        [Browsable(false)]
        public int FirstNumber
        {
            get { return firstNumber; }
            set { firstNumber = value; }
        }

        #endregion
        #region public int SecondNumber { get; set; }

        [NonSerialized]
        private int secondNumber = 0;

        [ InputVariable ]
        [Browsable(false)]
        public int SecondNumber
        {
            get { return secondNumber; }
            set { secondNumber = value; }
        }

        #endregion

        #region public TreatEqualAsEnum HowToTreatEqual { get; set; }

        public enum TreatEqualAsEnum
        {
            FirstNumberSmaller,
            SeparatePath,
            FirstNumberBigger
        }

        private TreatEqualAsEnum howToTreatEqual = TreatEqualAsEnum.SeparatePath;

        [ Category("2. Settings") ]
        public TreatEqualAsEnum HowToTreatEqual
        {
            get { return howToTreatEqual; }
            set
            {
                howToTreatEqual = value;
                if (howToTreatEqual == TreatEqualAsEnum.FirstNumberBigger)
                {
                    base.RemoveOutboundPath(PATH_SMALLER_OR_EQUAL);
                    base.RemoveOutboundPath(PATH_EQUAL);
                    base.RemoveOutboundPath(PATH_BIGGER);
                    base.AddOutboundPath(1, PATH_SMALLER);
                    base.AddOutboundPath(2, PATH_BIGGER_OR_EQUAL);
                }
                else if (howToTreatEqual == TreatEqualAsEnum.FirstNumberSmaller)
                {
                    base.RemoveOutboundPath(PATH_SMALLER);
                    base.RemoveOutboundPath(PATH_EQUAL);
                    base.RemoveOutboundPath(PATH_BIGGER_OR_EQUAL);
                    base.AddOutboundPath(1, PATH_SMALLER_OR_EQUAL);
                    base.AddOutboundPath(2, PATH_BIGGER);
                }
                else
                {
                    base.RemoveOutboundPath(PATH_SMALLER_OR_EQUAL);
                    base.RemoveOutboundPath(PATH_BIGGER_OR_EQUAL);
                    base.AddOutboundPath(1, PATH_SMALLER);
                    base.AddOutboundPath(2, PATH_EQUAL);
                    base.AddOutboundPath(3, PATH_BIGGER);
                }
            }
        }

        #endregion

        public override ComponentOutput Run(DataStorage storage)
        {
            int compare = this.firstNumber.CompareTo(this.secondNumber);

            string pathA, pathB, pathC;

            switch(this.howToTreatEqual)
            {
                case TreatEqualAsEnum.FirstNumberSmaller:
                    pathA = pathB = PATH_SMALLER_OR_EQUAL;
                    pathC = PATH_BIGGER;
                    break;
                case TreatEqualAsEnum.SeparatePath:
                    pathA = PATH_SMALLER;
                    pathB = PATH_EQUAL;
                    pathC = PATH_BIGGER;
                    break;
                case TreatEqualAsEnum.FirstNumberBigger:
                    pathA = PATH_SMALLER;
                    pathC = pathB = PATH_BIGGER_OR_EQUAL;
                    break;
                default:
                    throw new InvalidOperationException("Internal error.");
            }

            Guid retID = Guid.Empty;
            if (compare < 0)
                retID = base.GetPathID(pathA);
            else if (compare > 0)
                retID = base.GetPathID(pathC);
            else // if (compare == 0)
                retID = base.GetPathID(pathB);

            return new ComponentOutput(retID);
        }
    }
}
