using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;

using Softix.Drawing.Diagram.Core;

namespace Softix.Drawing.Diagram.Internals
{
    internal sealed class DiagramObjectPortInfo
    {
        public DiagramObjectPortInfo(
            DiagramObjectInfo parent,
            IDiagramObjectPort port,
            DiagramView view, Graphics g)
        {
            this.port = port;
            this.parent = parent;
            this.RefreshInfo(view, g);
        }

        #region public DiagramObjectInfo Parent { get; set; }

        private DiagramObjectInfo parent;

        public DiagramObjectInfo Parent
        {
            get { return parent; }
            set { parent = value; }
        }

        #endregion
        #region public IDiagramObjectPort Port { get; set; }

        private IDiagramObjectPort port;

        public IDiagramObjectPort Port
        {
            get { return port; }
            set { port = value; }
        }

        #endregion

        internal void RefreshInfo(DiagramView view, Graphics g)
        {
            Rectangle mainRect = this.parent.BoundRect;

            Size portNameSize = Size.Ceiling(g.MeasureString(port.Text, view.Font));
            mainRect.Width = Math.Max(mainRect.Width, portNameSize.Width + 6);

            Rectangle tmpRect = new Rectangle();
            tmpRect.Height = portNameSize.Height + 2;
            tmpRect.Y = mainRect.Bottom;
            mainRect.Height += tmpRect.Height;

            this.boundRect = tmpRect;
            this.parent.BoundRect = mainRect;
        }

        #region public Rectangle BoundRect { get; set; }

        private Rectangle boundRect;

        public Rectangle BoundRect
        {
            get { return boundRect; }
            set
            {
                boundRect = value;

                this.leftPort = new Rectangle(
                    boundRect.Left - 5,
                    boundRect.Top + (boundRect.Height - 7) / 2,
                    7, 7);
                this.rightPort = new Rectangle(
                    boundRect.Right - 2,
                    boundRect.Top + (boundRect.Height - 7) / 2,
                    7, 7);
            }
        }

        #endregion
        #region public Rectangle LeftPort { get; }

        private Rectangle leftPort;

        public Rectangle LeftPort
        {
            get { return leftPort; }
        }

        #endregion
        #region public Rectangle RightPort { get; }

        private Rectangle rightPort;

        public Rectangle RightPort
        {
            get { return rightPort; }
        }

        #endregion
    }
}
