using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

using Softix.Drawing.Diagram.Core;

namespace Softix.Drawing.Diagram.Internals
{
    internal sealed class DiagramObjectTarget
    {
        private Rectangle bounds = Rectangle.Empty;
        private Rectangle imageBounds = Rectangle.Empty;

        public DiagramObjectTarget(DiagramObjectInfo info)
        {
            this.diagramObject = info.DiagramObject;
            this.bounds = info.BoundRect;
            this.imageBounds = info.ImageRect;
            this.diagramObjectEdge = DiagramObjectEdge.Top;
        }

        internal void RefreshInfo(Point currentLocation)
        {
            float
                x11 = this.bounds.Left,
                x12 = this.bounds.Right,
                y11 = this.bounds.Top,
                y12 = this.bounds.Bottom;
            float
                x21 = this.bounds.Left,
                x22 = this.bounds.Right,
                y21 = this.bounds.Bottom,
                y22 = this.bounds.Top;

            //  y = a*x + b
            //  -----
            //  y11 = a*x11 + b
            //  y12 = a*x12 + b
            //  -----
            //  b = y11 - a*x11;
            //  y12 = a*x12 + y11 - a*x11
            float
                a1 = (y12 - y11) / (x12 - x11),
                b1 = y11 - a1 * x11;
            float
                a2 = (y22 - y21) / (x22 - x21),
                b2 = y21 - a2 * x21;
            float
                y = currentLocation.Y,
                y1 = a1 * (float)currentLocation.X + b1,
                y2 = a2 * (float)currentLocation.X + b2;
            //  \   A   /  A - y > y1
            //   \     /     - y > y2
            //    \   /    
            //     \ /     B - y > y1
            //  D   X   B    - y < y2
            //     / \     
            //    /   \    C - y < y1
            //   /     \     - y < y2
            //  /   C   \  
            //             D - y < y1
            //               - y > y2
            if ((y > y1) && (y > y2))
                this.diagramObjectEdge = DiagramObjectEdge.Bottom;
            else if ((y > y1) && (y < y2))
                this.diagramObjectEdge = DiagramObjectEdge.Left;
            else if ((y < y1) && (y < y2))
                this.diagramObjectEdge = DiagramObjectEdge.Top;
            else if ((y < y1) && (y > y2))
                this.diagramObjectEdge = DiagramObjectEdge.Right;

            switch (this.diagramObjectEdge)
            {
                case DiagramObjectEdge.Top:
                    this.targetPoint = new Point(
                        this.bounds.Left + this.bounds.Width / 2,
                        this.bounds.Top - 1);
                    break;
                case DiagramObjectEdge.Bottom:
                    this.targetPoint = new Point(
                        this.bounds.Left + this.bounds.Width / 2,
                        this.bounds.Bottom + 1);
                    break;
                case DiagramObjectEdge.Left:
                    this.targetPoint = new Point(
                        this.bounds.Left - 1,
                        this.imageBounds.Top + this.imageBounds.Height / 2);
                    break;
                case DiagramObjectEdge.Right:
                    this.targetPoint = new Point(
                        this.bounds.Right + 1,
                        this.imageBounds.Top + this.imageBounds.Height / 2);
                    break;
                default:
                    this.targetPoint = Point.Empty;
                    break;
            }
        }

        #region public IDiagramObject DiagramObject { get; }

        private IDiagramObject diagramObject;

        public IDiagramObject DiagramObject
        {
            get { return diagramObject; }
        }

        #endregion
        #region public DiagramObjectEdge DiagramObjectEdge { get; }

        private DiagramObjectEdge diagramObjectEdge;

        public DiagramObjectEdge DiagramObjectEdge
        {
            get { return diagramObjectEdge; }
        }

        #endregion

        #region public Point TargetPoint { get; }

        private Point targetPoint;

        public Point TargetPoint
        {
            get { return targetPoint; }
        }

        #endregion
    }
}
