using System;
using System.Collections.Generic;
using System.Text;

using Softix.Drawing.Diagram.Core;

namespace Softix.Drawing.Diagram.Internals
{
    internal interface IHoverObjectInfo
    {
        bool IsMatch(IDiagramEntity entity, params object[] extras);
        object InnerEntity { get; }
    }

    internal sealed class HoverDummyInfo : IHoverObjectInfo
    {
        public HoverDummyInfo()
        { }

        public bool IsMatch(IDiagramEntity entity, params object[] extras)
        {
            return false;
        }

        public object InnerEntity { get { return null; } }
    }

    internal sealed class HoverDiagramObjectInfo : IHoverObjectInfo
    {
        public HoverDiagramObjectInfo(IDiagramObject diagramObject)
        {
            this.diagramObject = diagramObject;
        }

        #region public IDiagramObject DiagramObject { get; }

        private IDiagramObject diagramObject;

        public IDiagramObject DiagramObject
        {
            get { return diagramObject; }
        }

        #endregion

        #region IHoverObjectInfo Members

        public bool IsMatch(IDiagramEntity entity, params object[] extras)
        {
            if ((entity == this.diagramObject) &&
                ((extras == null) || (extras.Length == 0)))
            {
                return true;
            }
            return false;
        }

        public object InnerEntity { get { return this.diagramObject; } }

        #endregion

        #region Equals

        public override bool Equals(object obj)
        {
            if (obj is HoverDiagramObjectInfo)
            {
                HoverDiagramObjectInfo hover = (HoverDiagramObjectInfo)obj;
                return (this.diagramObject == hover.diagramObject);
            }
            return false;
        }

        public override int GetHashCode()
        {
            return this.diagramObject.GetHashCode();
        }

        #endregion
    }

    internal sealed class HoverDiagramObjectPortInfo : IHoverObjectInfo
    {
        public HoverDiagramObjectPortInfo(
            IDiagramObject diagramObject,
            IDiagramObjectPort port,
            DiagramObjectPortAlign portAlign)
        {
            this.diagramObject = diagramObject;
            this.port = port;
            this.portAlign = portAlign;
        }

        #region public IDiagramObject DiagramObject { get; }

        private IDiagramObject diagramObject;

        public IDiagramObject DiagramObject
        {
            get { return diagramObject; }
        }

        #endregion
        #region public IDiagramObjectPort Port { get; }

        private IDiagramObjectPort port;

        public IDiagramObjectPort Port
        {
            get { return port; }
        }

        #endregion
        #region public DiagramObjectPortAlign PortAlign { get; }

        private DiagramObjectPortAlign portAlign;

        public DiagramObjectPortAlign PortAlign
        {
            get { return portAlign; }
        }

        #endregion

        #region IHoverObjectInfo Members

        public bool IsMatch(IDiagramEntity entity, params object[] extras)
        {
            if ((entity == this.port) &&
                (extras != null) && (extras.Length == 1) &&
                (extras[0] is DiagramObjectPortAlign) &&
                ((DiagramObjectPortAlign)extras[0] == this.portAlign))
            {
                return true;
            }
            return false;
        }

        public object InnerEntity { get { return this.diagramObject; } }

        #endregion

        #region Equals

        public override bool Equals(object obj)
        {
            if (obj is HoverDiagramObjectPortInfo)
            {
                HoverDiagramObjectPortInfo hover = (HoverDiagramObjectPortInfo)obj;
                return (
                    (this.diagramObject == hover.diagramObject) &&
                    (this.port == hover.port) &&
                    (this.portAlign == hover.portAlign)
                    );
            }
            return false;
        }

        public override int GetHashCode()
        {
            return (this.port.GetHashCode() | (int)this.portAlign);
        }

        #endregion
    }

    internal sealed class HoverDiagramObjectArtifactInfo : IHoverObjectInfo
    {
        #region IHoverObjectInfo Members

        public bool IsMatch(IDiagramEntity entity, params object[] extras)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public object InnerEntity { get { return null; } }

        #endregion
    }
}
