using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;

using Softix.Drawing.Diagram.Core;

namespace Softix.Drawing.Diagram.Simple
{
    public class SimpleDiagramObject : DiagramEntityBase, IDiagramObject
    {
        public SimpleDiagramObject()
        { }

        public SimpleDiagramObject(string name, string text, Point location)
        {
            this.name = name;
            base.Text = text;
            this.location = location;
        }

        #region public Point Location { get; set; }

        private Point location;

        public Point Location
        {
            get { return location; }
            set
            {
                location = value;
                this.Redraw();
            }
        }

        #endregion
        #region public string Name { get; set; }

        private string name;

        public string Name
        {
            get { return name; }
            set
            {
                name = value;
                this.Redraw();
            }
        }

        #endregion

        #region ports

        protected List<IDiagramObjectPort> ports = null;

        public IDiagramObjectPort[] Ports
        {
            get
            {
                if (this.ports == null)
                    return new IDiagramObjectPort[0];
                else
                {
                    this.ports.Sort(
                        delegate(IDiagramObjectPort a, IDiagramObjectPort b)
                        {
                            int ret = a.Index.CompareTo(b.Index);
                            if (ret == 0)
                                ret = a.Text.CompareTo(b.Text);
                            return ret;
                        });
                    return this.ports.ToArray();
                }
            }
        }

        protected void ClearPorts()
        {
            if (this.ports == null)
                this.ports = new List<IDiagramObjectPort>();
            else
                this.ports.Clear();
            this.Redraw();
        }

        public IDiagramObjectPort AddPort(int index, string text)
        {
            return this.AddPort(new SimpleDiagramObjectPort(index, text));
        }

        public IDiagramObjectPort AddPort(IDiagramObjectPort port)
        {
            if (this.ports == null)
                this.ports = new List<IDiagramObjectPort>();
            this.ports.Add(port);

            this.Redraw();
            return port;
        }

        #endregion
        #region artifacts

        private Dictionary<String, IDiagramArtifact> artifacts = null;

        public void ClearArtifacts()
        {
            if (this.artifacts == null)
                this.artifacts = new Dictionary<string, IDiagramArtifact>();
            else
                this.artifacts.Clear();
            this.Redraw();
        }

        public void AddArtifact(string key, IDiagramArtifact artifact)
        {
            if (this.artifacts == null)
                this.artifacts = new Dictionary<string, IDiagramArtifact>();
            this.artifacts[key] = artifact;
            this.Redraw();
        }

        public void RemoveArtifact(string key)
        {
            if ((this.artifacts != null) &&
                (this.artifacts.ContainsKey(key) == true))
            {
                this.artifacts.Remove(key);
                this.Redraw();
            }
        }

        public IDiagramArtifact[] Artifacts
        {
            get
            {
                if (this.artifacts == null)
                    return new IDiagramArtifact[0];

                IDiagramArtifact[] artifacts = new IDiagramArtifact[this.artifacts.Count];
                this.artifacts.Values.CopyTo(artifacts, 0);
                return artifacts;
            }
        }

        #endregion

        public event EventHandler ShouldRedraw;

        protected void Redraw()
        {
            if (this.ShouldRedraw != null)
                this.ShouldRedraw(this, EventArgs.Empty);
        }
    }
}
