using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Core.Components
{
    [Serializable]
    public class SimpleComponentLink : ISOComponentLink
    {
        public SimpleComponentLink()
        {
            this.id = Guid.NewGuid();
        }

        #region public Guid ID { get; set; }

        private Guid id = Guid.Empty;

        public Guid ID
        {
            get { return id; }
            protected set { id = value; }
        }

        #endregion

        #region public ISOModel ParentModel { get; set; }

        private ISOModel parentModel = null;

        public ISOModel ParentModel
        {
            get { return parentModel; }
            set { parentModel = value; }
        }

        #endregion

        #region source

        private ISOComponent sourceComponent = null;
        private ISOComponentOutboundPath sourceComponentPath = null;

        public ISOComponent SourceComponent
        {
            get { return this.sourceComponent; }
        }

        public ISOComponentOutboundPath SourceComponentPath
        {
            get { return this.sourceComponentPath; }
        }

        public void SetSourceComponent(ISOComponent component, Guid componentPathID)
        {
            if (this.parentModel != null)
            {
                // validate source component on model
                ISOComponent testComp = this.parentModel.GetComponent(component.ID);
                if (testComp == null)
                    throw new InvalidOperationException("Source component not found on model");
            }

            // validate outbound path
            ISOComponentOutboundPath testPath = Array.Find<ISOComponentOutboundPath>(
                component.OutboundPaths,
                delegate(ISOComponentOutboundPath tmpPath)
                {
                    return (tmpPath.ID == componentPathID);
                });
            if (testPath == null)
                throw new InvalidOperationException("Source path not found.");

            // everything looks ok
            this.sourceComponent = component;
            this.sourceComponentPath = testPath;
        }

        #endregion

        #region target

        private ISOComponent targetComponent;

        public ISOComponent TargetComponent
        {
            get { return targetComponent; }
            set
            {
                if (this.parentModel != null)
                {
                    // validate target component on model
                    ISOComponent testComp = this.parentModel.GetComponent(value.ID);
                    if (testComp == null)
                        throw new InvalidOperationException("Target component not found on model");
                }

                this.targetComponent = value;
            }
        }

        #endregion
    }
}
