using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data
{
    public sealed class DataStorage
    {
        private Dictionary<Guid, List<VariableValue>> storage = null;

        public DataStorage()
        {
            this.storage = new Dictionary<Guid, List<VariableValue>>();
        }

        public void AddVariable(Guid compID, String varName, object varValue)
        {
            List<VariableValue> list = null;

            if (this.storage.ContainsKey(compID) == true)
            {
                list = this.storage[compID];
            }

            if (list == null)
            {
                list = new List<VariableValue>();
                this.storage.Add(compID, list);
            }

            VariableValue vv = list.Find(
                delegate(VariableValue tmpVV)
                {
                    return (tmpVV.Name == varName);
                });
            if (vv == null)
                list.Add(new VariableValue(varName, varValue));
            else
                vv.Value = varValue;
        }

        public Object GetVariableValue(Variable var)
        {
            Guid compID = var.SourceComponentID;
            if (this.storage.ContainsKey(compID) == false)
                return null;

            List<VariableValue> list = this.storage[compID];
            VariableValue value = list.Find(
                delegate(VariableValue tmpValue)
                {
                    return (tmpValue.Name == var.Name);
                });

            return (value == null) ? null : value.Value;
        }

        public VariableValue[] GetOutputForComponent(Guid compID)
        {
            if (this.storage.ContainsKey(compID) == false)
                return new VariableValue[0];

            return this.storage[compID].ToArray();
        }

        public Guid[] GetComponentIDs()
        {
            List<Guid> list = new List<Guid>();
            list.AddRange(this.storage.Keys);
            return list.ToArray();
        }
    }
}
