using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.ComponentModel;

using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Core.Data
{
    [AttributeUsage(
        AttributeTargets.Property,
        Inherited = true,
        AllowMultiple = false
        )]
    public sealed class InputVariableAttribute : Attribute
    {
        public InputVariableAttribute()
        { }

        public InputVariableAttribute(string description)
        {
            this.description = description;
        }

        #region public String Description { get; set; }

        private String description = String.Empty;

        public String Description
        {
            get { return description; }
            set { description = value; }
        }

        #endregion

        public static Variable[] GetInputDataDefinitions(ISOComponent component)
        {
            VariableList list = new VariableList();

            foreach (PropertyInfo property in component.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                object[] attrs = property.GetCustomAttributes(typeof(InputVariableAttribute), true);
                if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is InputVariableAttribute))
                {
                    InputVariableAttribute attr = (InputVariableAttribute)attrs[0];

                    Variable def = new Variable();
                    def.Model = component.ParentModel;
                    def.Name = property.Name;
                    def.Type = property.PropertyType;
                    def.SourceComponentID = component.ID;
                    list.Add(def);
                }
            }

            return list.ToArray();
        }

        public static void SetInputVariables(ISOComponent component, DataStorage storage)
        {
            foreach (PropertyInfo property in component.GetType().GetProperties(BindingFlags.Instance | BindingFlags.Public))
            {
                object[] attrs = property.GetCustomAttributes(typeof(InputVariableAttribute), true);
                if ((attrs != null) && (attrs.Length == 1) && (attrs[0] is InputVariableAttribute))
                {
                    InputVariableAttribute attr = (InputVariableAttribute)attrs[0];

                    VariableMapping mapping = component.InputDataMappings.Find(
                        delegate(VariableMapping tmpMapping)
                        {
                            return (tmpMapping.Variablename == property.Name);
                        });

                    if ((mapping == null) || (mapping.Mapper == null))
                        throw new InvalidOperationException("No mapping found for variable '" + property.Name + "' on component '" + component.Name + "'");

                    object value = mapping.Mapper.BuildData(storage);
                    if (value == null)
                    {
                        // value is null... what if it's a value type?...
                        if (property.PropertyType.IsValueType == true)
                        {
                            PropertyDescriptor pd = TypeDescriptor.GetProperties(property.DeclaringType).Find(property.Name, false);
                            if ((pd != null) && (pd.Converter != null) && (pd.Converter.GetCreateInstanceSupported() == true))
                                property.SetValue(component, pd.Converter.CreateInstance(null), null);
                            else
                                property.SetValue(component, 0 /* ??? */, null);
                        }
                        else
                            property.SetValue(component, null, null);
                    }
                    else
                    {
                        // value is not null... :)
                        if (property.PropertyType.IsAssignableFrom(value.GetType()) == false)
                            throw new InvalidCastException("Wrong value type for variable '" + property.Name + "' on component '" + component.Name + "'");

                        property.SetValue(component, value, null);
                    }
                }
            }
        }
    }
}
