using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data
{
    internal sealed class ModelDataStorage
    {
        private ISODataStorage publicData = null;
        private Dictionary<Guid, ISODataStorage> privateData = null;

        internal ModelDataStorage(Dictionary<String, Object> initialData)
        {
            if (initialData != null)
            {
                this.publicData = new SimpleDataStorage();
                foreach (KeyValuePair<String, Object> pair in initialData)
                {
                    if (pair.Value == null)
                        continue;

                    this.publicData.SetValue(pair.Key, pair.Value);
                }
            }
        }

        internal ISODataStorage PublicData
        {
            get
            {
                if (this.publicData == null)
                    this.publicData = new SimpleDataStorage();

                return this.publicData;
            }
        }

        internal ISODataStorage GetComponentPrivateData(Guid componentID)
        {
            if (this.privateData == null)
                this.privateData = new Dictionary<Guid, ISODataStorage>();

            if (this.privateData.ContainsKey(componentID) == false)
            {
                ISODataStorage data = new SimpleDataStorage();
                this.privateData.Add(componentID, data);
            }

            return this.privateData[componentID];
        }

        internal ISOData GetComponentData(Guid componentID)
        {
            return new SimpleData(
                this.GetComponentPrivateData(componentID),
                this.PublicData);
        }
    }
}
