using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data
{
    internal sealed class SimpleData : ISOData
    {
        private ISODataStorage privateData = null;
        private ISODataStorage publicData = null;

        internal SimpleData(ISODataStorage privateData, ISODataStorage publicData)
        {
            this.privateData = privateData;
            this.publicData = publicData;
        }

        #region ISOData Members

        public ISODataStorage PublicData
        {
            get { return this.publicData; }
        }

        public ISODataStorage PrivateData
        {
            get { return this.privateData; }
        }

        #endregion

        #region ISODataStorage Members (proxy to publicData)

        public T GetValue<T>(string varName)
        {
            return this.publicData.GetValue<T>(varName);
        }

        public object GetValue(string varName)
        {
            return this.publicData.GetValue(varName);
        }

        public void SetValue<T>(string varName, T varValue)
        {
            this.publicData.SetValue<T>(varName, varValue);
        }

        public void SetValue(string varName, object varValue)
        {
            this.publicData.SetValue(varName, varValue);
        }

        public string[] VariableNames
        {
            get
            {
                return this.publicData.VariableNames;
            }
        }

        public bool HasVariable(string varName, Type varType)
        {
            return this.publicData.HasVariable(varName, varType);
        }

        #endregion
    }
}
