using System;
using System.Collections.Generic;
using System.Text;

namespace Softix.Orchestration.Core.Data
{
    internal class SimpleDataStorage : ISODataStorage
    {
        private Dictionary<string, object> storage = new Dictionary<string, object>();

        internal SimpleDataStorage()
        {
        }

        #region get data

        public T GetValue<T>(string varName)
        {
            try
            {
                object gigi = this.GetValue(varName);
                if (typeof(T).IsAssignableFrom(gigi.GetType()) == true)
                    return (T)gigi;
                else
                    throw new DataStorageException(
                        "Variable '" + varName + "' is not assignable to type '" +
                        typeof(T).Name + "'.");
            }
            catch (Exception ex)
            {
                throw new DataStorageException(
                    "Error loading variable '" + varName + "' from storage",
                    ex);
            }
        }

        public object GetValue(String varName)
        {
            if (this.storage.ContainsKey(varName) == true)
            {
                object gigi = this.storage[varName];
                return gigi;
            }
            else
                throw new VariableNotFoundException(varName);
        }

        #endregion

        #region set data

        public void SetValue<T>(string varName, T varValue)
        {
            this.SetValue(varName, varValue);
        }

        public void SetValue(String varName, object varValue)
        {
            try
            {
                if ((varValue == null) && (this.storage.ContainsKey(varName) == true))
                {
                    // remove variable
                    this.storage.Remove(varName);
                }
                else if (varValue != null)
                {
                    // test variable type
                    if (this.storage.ContainsKey(varName) == true)
                    {
                        object value = this.storage[varName];
                        if ((value != null) &&
                            (value.GetType().IsAssignableFrom(varValue.GetType()) == false))
                            throw new DataStorageException("You cannot change a variable's type.");
                    }

                    // add variable
                    this.storage[varName] = varValue;
                }
            }
            catch (Exception ex)
            {
                throw new DataStorageException(
                    "Error saving variable '" + varName + "' to storage",
                    ex);
            }
        }

        #endregion

        public string[] VariableNames
        {
            get
            {
                List<string> keys = new List<string>(this.storage.Keys);
                keys.Sort();
                return keys.ToArray();
            }
        }

        public bool HasVariable(string varName, Type varType)
        {
            if (this.storage.ContainsKey(varName) == false)
                return false;
            else
            {
                object value = this.storage[varName];
                return varType.IsAssignableFrom(value.GetType());
            }
        }
    }
}
