using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Validation;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Core.Data
{
    [Serializable]
    public sealed class Variable : ValidationBase, ISOSupportsValidation
    {
        public Variable()
        { }

        public Variable(ISOModel model, Guid sourceComponentID, String name, Type type)
        {
            this.model = model;
            this.sourceComponentID = sourceComponentID;
            this.name = name;
            this.type = type;
        }

        #region public ISOModel Model { get; set; }

        private ISOModel model;

        public ISOModel Model
        {
            get { return model; }
            set { model = value; }
        }

        #endregion
        #region public Guid SourceComponentID { get; set; }

        private Guid sourceComponentID;

        public Guid SourceComponentID
        {
            get { return sourceComponentID; }
            set
            {
                sourceComponentID = value;
                base.RunValidation();
            }
        }

        #endregion
        #region public String Name { get; set; }

        private String name;

        public String Name
        {
            get { return name; }
            set
            {
                name = value;
                base.RunValidation();
            }
        }

        #endregion

        #region public Type Type { get; set; }

        private Type type = null;

        public Type Type
        {
            get { return this.type; }
            set
            {
                this.type = value;
                base.RunValidation();
            }
        }

        #endregion
        #region public bool IsArray { get; }

        public bool IsArray
        {
            get { return (this.type == null) ? false : this.type.IsArray; }
        }

        #endregion

        #region ISOSupportsValidation Members

        private IEnumerable<ValidationBreak> BaseValidationBreaks
        {
            get { return base.GetValidationBreaks(); }
        }

        protected override IEnumerable<ValidationBreak> GetValidationBreaks()
        {
            // add base breaks
            foreach (ValidationBreak vb in this.BaseValidationBreaks)
                yield return vb;

            if (String.IsNullOrEmpty(this.name) == true)
                yield return new ValidationBreak(this,
                    "Property 'name' cannot be empty.",
                    "You must specify a name for this data definition.");

            if ((this.type == null) || (this.type == Type.Missing))
                yield return new ValidationBreak(this,
                    "Property 'type' cannot be null.",
                    "You must specify a type for this data definition.");

        }

        #endregion
    }
}
