using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Data.Mappers;
using Softix.Orchestration.Core.Validation;

namespace Softix.Orchestration.Core.Data
{
    [Serializable]
    public sealed class VariableMapping : ValidationBase
    {
        public VariableMapping()
        { }

        public VariableMapping(String variableName)
        {
            this.variableName = variableName;
        }

        #region public String Variablename { get; set; }

        private String variableName;

        public String Variablename
        {
            get { return variableName; }
            set { variableName = value; }
        }

        #endregion
        #region public IVariableMapper Mapper { get; set; }

        private IVariableMapper mapper;

        public IVariableMapper Mapper
        {
            get { return mapper; }
            set { mapper = value; }
        }

        #endregion

        private IEnumerable<ValidationBreak> BaseValidationBreaks
        {
            get { return base.GetValidationBreaks(); }
        }

        protected override IEnumerable<ValidationBreak> GetValidationBreaks()
        {
            // parent reasons
            foreach (ValidationBreak vb in this.BaseValidationBreaks)
                yield return vb;

            if (this.mapper == null)
            {
                yield return new ValidationBreak(this,
                    "Mapper not set for variable '" + this.variableName + "'.",
                    "You must have a variable mapper for each input variable of a component.",
                    "Mapper");
            }
        }

        public override string ToString()
        {
            return "Mapping: " + this.variableName;
        }
    }
}
