using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Models.Events;

namespace Softix.Orchestration.Core.Models
{
    public interface ISOModel
    {
        String Name { get; set; }

        // add & remove components
        void AddComponent(ISOComponent component);
        void RemoveComponent(Guid componentID);
        void RemoveComponent(ISOComponent component);
        ISOComponent GetComponent(Guid componentID);

        event ModelComponentHandler ComponentAdded;
        event ModelComponentHandler ComponentRemoved;
        event ModelComponentHandler ComponentChanged;

        // add & remove component links
        void AddComponentLink(ISOComponentLink link);
        void RemoveComponentLink(Guid linkID);
        void RemoveComponentLink(ISOComponentLink link);
        ISOComponentLink GetComponentLink(Guid linkID);

        event ModelComponentLinkHandler ComponentLinkAdded;
        event ModelComponentLinkHandler ComponentLinkRemoved;
        event ModelComponentLinkHandler ComponentLinkChanged;

        ISOComponentLink[] GetOutboundLinksForComponent(Guid componentID);
        ISOComponentLink[] GetOutboundLinksForComponent(ISOComponent component);
        ISOComponentLink[] GetOutboundLinksForComponent(Guid componentID, Guid pathID);
        ISOComponentLink[] GetOutboundLinksForComponent(ISOComponent component, ISOComponentOutboundPath path);
        ISOComponentLink[] GetInboundLinksForComponent(Guid componentID);
        ISOComponentLink[] GetInboundLinksForComponent(ISOComponent component);

        // data definitions
        Variable[] InitialVariables { get; set; }
        Variable[] FinalVariables { get; set; }
        event ModelDataChanged InitialVariablesChanged;
        event ModelDataChanged FinalVariablesChanged;

        Variable[] GetVariablesForComponentLink(Guid linkID);
        Variable[] GetVariablesForComponentLink(ISOComponentLink link);
        Variable[] GetVariablesForComponent(Guid componentID);
        Variable[] GetVariablesForComponent(ISOComponent component);

        // runtime
        ISOComponent[] StartComponents { get; }
        DataStorage Run(ISOComponent start);
    }
}
