using System;
using System.Collections.Generic;
using System.Text;

using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Core.Models
{
    [ Serializable ]
    public sealed class SimpleExecutionModel : ModelBase
    {
        public override DataStorage Run(ISOComponent start)
        {
            DataStorage storage = new DataStorage();

            ISOComponent current = start;
            while (current != null)
            {
                // run component
                InputVariableAttribute.SetInputVariables(current, storage);
                ComponentOutput cOut = current.Run();

                // get path
                ISOComponentOutboundPath path = Array.Find<ISOComponentOutboundPath>(
                    current.OutboundPaths,
                    delegate(ISOComponentOutboundPath tmpPath)
                    {
                        return (tmpPath.ID == cOut.OutboundPathID);
                    });

                if (path != null)
                {
                    #region add variable values

                    Variable[] vars = OutputVariableAttribute.GetOutputDataDefinitions(
                        current, path);
                    if ((vars != null) && (vars.Length > 0))
                    {
                        List<VariableValue> list = new List<VariableValue>();

                        // add attribute-based output vars to cOut
                        OutputVariableAttribute.AddToComponentOutput(current, path, cOut);
                        // add variables
                        foreach (Variable var in vars)
                        {
                            object value = null;
                            try { value = cOut[var.Name]; }
                            catch { value = null; }

                            storage.AddVariable(current.ID, var.Name, value);
                        }
                    }

                    #endregion

                    // get next component
                    ISOComponentLink[] links = base.GetOutboundLinksForComponent(current, path);
                    if ((links == null) || (links.Length == 0))
                        current = null;
                    else if (links.Length != 1)
                        throw new InvalidOperationException("Incorrect number of outbound links for component '" + current.Name + "' found on path '" + path.Name + "'.");
                    else
                        current = links[0].TargetComponent;
                }
                else
                    current = null;
            }

            return storage;
        }
    }
}
