using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.ComponentModel;

namespace Softix.Orchestration.Core.Validation
{
    [ Serializable ]
    public abstract class ValidationBase : ISOSupportsValidation
    {
        [ NonSerialized ]
        private ValidationBreak[] validationBreaks;

        public ValidationBase()
        {
            this.validationBreaks = null;
        }

        protected virtual IEnumerable<ValidationBreak> GetValidationBreaks()
        {
            yield break;
        }

        #region ISOSupportsValidation

        [ Category("1. Information") ]
        [Browsable(false)]
        public bool IsValid
        {
            get { return ((this.validationBreaks == null) || (this.validationBreaks.Length == 0)); }
        }

        [ Browsable(false) ]
        public ValidationBreak[] ValidationBreaks
        {
            get { return (this.validationBreaks == null) ? new ValidationBreak[0] : this.validationBreaks; }
        }

        public ValidationBreak[] RunValidation()
        {
            List<ValidationBreak> result = new List<ValidationBreak>();

            IEnumerable<ValidationBreak> enumerable = this.GetValidationBreaks();
            if (enumerable != null)
            {
                result.AddRange(enumerable);
            }

            bool oldValid = this.IsValid;
            this.validationBreaks = result.ToArray();
            if (this.IsValid != oldValid)
                this.OnValidationChanged();
            return this.ValidationBreaks;
        }

        #region event

        [field: NonSerialized]
        public event ValidationHandler ValidationChanged;

        protected void OnValidationChanged()
        {
            if (this.ValidationChanged != null)
                this.ValidationChanged(this);
        }

        #endregion

        #endregion
    }
}
