using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Models;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.DefaultComponents.InsertData
{
    [Serializable]
    public sealed class InsertNumbers : ComponentBase, IHasDynamicOutputData
    {
        public const string PATH_NUMBERS = "Numbers";
        public const string VAR_NUMBERS = "Numbers";

        public InsertNumbers()
        {
            base.AddOutboundPath(PATH_NUMBERS);
            base.Name = "Insert Numbers";
        }

        #region IHasDynamicOutputData Members

        public Variable[] GetDynamicOutputDataDefinitions(string pathName)
        {
            VariableList list = new VariableList();

            if (pathName == PATH_NUMBERS)
            {
                Type type = null;
                switch (this.numbersType)
                {
                    case InsertNumbersOutputType.Integer:
                        type = typeof(Int32[]);
                        break;
                    case InsertNumbersOutputType.Decimal:
                        type = typeof(Decimal[]);
                        break;
                    case InsertNumbersOutputType.Float:
                        type = typeof(Single[]);
                        break;
                    default:
                        throw new InvalidOperationException("Should never happen.");
                }

                list.Add(new Variable(base.ParentModel, base.ID, VAR_NUMBERS, type));
            }

            return list.ToArray();
        }

        #endregion

        #region public InsertNumbersOutputType NumbersType { get; set; }

        public enum InsertNumbersOutputType
        {
            Integer,
            Decimal,
            Float
        }

        private InsertNumbersOutputType numbersType = InsertNumbersOutputType.Integer;

        [ Category("2. Settings") ]
        public InsertNumbersOutputType NumbersType
        {
            get { return numbersType; }
            set
            {
                numbersType = value;
                base.OnComponentChanged();
            }
        }

        #endregion
        #region public String Numbers { get; set; }

        [NonSerialized]
        private String numbers = String.Empty;

        [ InputVariable ]
        [ Browsable(false) ]
        public String Numbers
        {
            get { return numbers; }
            set { numbers = value; }
        }

        #endregion

        public override ComponentOutput Run()
        {
            ArrayList list = new ArrayList();
            Type type = null;
            switch (this.numbersType)
            {
                case InsertNumbersOutputType.Integer:
                    type = typeof(Int32);
                    break;
                case InsertNumbersOutputType.Decimal:
                    type = typeof(Decimal);
                    break;
                case InsertNumbersOutputType.Float:
                    type = typeof(Single);
                    break;
                default:
                    throw new InvalidOperationException("Should never happen.");
            }

            string[] parts = (this.numbers == null)
                ? new string[0]
                : this.numbers.Split(',');
            foreach (string part in parts)
            {
                try { list.Add(Convert.ChangeType(part.Trim(), type)); }
                catch { /* ignore */ }
            }

            ComponentOutput output = new ComponentOutput(base.GetPathID(PATH_NUMBERS));
            output[VAR_NUMBERS] = list.ToArray(type);
            return output;
        }
    }
}
