using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Xml;

using Softix.Drawing.Diagram.Core;
using Softix.Drawing.Diagram.Simple;
using Softix.Orchestration.Core.Components;

namespace Softix.Orchestration.Editor.Diagram
{
    public sealed class OrchestrationObjectLink : SimpleDiagramLink, IXmlSerializable
    {
        public OrchestrationObjectLink()
            : base()
        { }

        public OrchestrationObjectLink(IDiagram diagram, ISOComponentLink link)
            : this()
        {
            this.diagram = diagram;
            this.Link = link;
        }

        #region public IDiagram Diagram { get; set; }

        private IDiagram diagram;

        public IDiagram Diagram
        {
            get { return diagram; }
            set { diagram = value; }
        }

        #endregion
        #region public ISOComponentLink Link { get; set; }

        private ISOComponentLink link;

        public ISOComponentLink Link
        {
            get { return link; }
            set
            {
                this.link = value;
                IDiagramObject source = Array.Find<IDiagramObject>(this.diagram.DiagramObjects,
                    delegate(IDiagramObject tmp)
                    {
                        return ((tmp is OrchestrationObject) &&
                            (((OrchestrationObject)tmp).Component.ID == link.SourceComponent.ID));
                    });
                if (source == null)
                    throw new InvalidOperationException("Source component not found on diagram.");
                IDiagramObjectPort sourcePort = Array.Find<IDiagramObjectPort>(source.Ports,
                    delegate(IDiagramObjectPort tmp)
                    {
                        return ((tmp is OrchestrationObjectPort) &&
                            (((OrchestrationObjectPort)tmp).Path.ID == link.SourceComponentPath.ID));
                    });
                if (sourcePort == null)
                    throw new InvalidOperationException("Source port not found on diagram.");
                IDiagramObject target = Array.Find<IDiagramObject>(this.diagram.DiagramObjects,
                    delegate(IDiagramObject tmp)
                    {
                        return ((tmp is OrchestrationObject) &&
                            (((OrchestrationObject)tmp).Component.ID == link.TargetComponent.ID));
                    });
                if (target == null)
                    throw new InvalidOperationException("Target component not found on diagram.");

                base.Start = source;
                base.StartPort = sourcePort;
                base.End = target;
            }
        }

        #endregion

        #region IXmlSerializable Members

        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(XmlReader r)
        {
        }

        public void WriteXml(XmlWriter w)
        {
            w.WriteStartElement("Link");
            {
                w.WriteAttributeString("LinkID", this.link.ID.ToString());

                w.WriteStartElement("LinkSource");
                {
                    w.WriteAttributeString("ComponentName", this.link.SourceComponent.Name);
                    w.WriteAttributeString("ComponentID", this.link.SourceComponent.ID.ToString());

                    w.WriteStartElement("SourcePort");
                    w.WriteValue(this.link.SourceComponentPath.ID.ToString());
                    w.WriteEndElement();

                    w.WriteStartElement("SourcePortLocation");
                    w.WriteValue(base.StartPortAlign.ToString());
                    w.WriteEndElement();
                }
                w.WriteEndElement();

                w.WriteStartElement("LinkTarget");
                {
                    w.WriteAttributeString("ComponentName", this.link.TargetComponent.Name);
                    w.WriteAttributeString("ComponentID", this.link.TargetComponent.ID.ToString());

                    w.WriteStartElement("TargetEdge");
                    w.WriteValue(base.EndEdge.ToString());
                    w.WriteEndElement();
                }
                w.WriteEndElement();
            }
            w.WriteEndElement();
        }

        #endregion
    }
}
