using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Validation;
using Softix.Orchestration.Editor.Editors.MappingEditors;

namespace Softix.Orchestration.Editor.Editors
{
    public partial class ComponentEditor : Form
    {
        public ComponentEditor()
        {
            InitializeComponent();
        }

        public ComponentEditor(ISOComponent component)
            : this()
        {
            this.Component = component;
        }

        #region public ISOComponent Component { get; set; }

        private ISOComponent component;

        public ISOComponent Component
        {
            get { return component; }
            set
            {
                component = value;
                this.pGrid.SelectedObject = component;
                this.RefreshInputMappings();
            }
        }

        #endregion

        private void RefreshInputMappings()
        {
            this.lvwInputMappings.Items.Clear();
            this.splitInputMappings.Panel2.Controls.Clear();

            if (this.component != null)
            {
                Variable[] vars = InputVariableAttribute.GetInputDataDefinitions(this.component);
                if ((vars != null) && (vars.Length != 0))
                {
                    foreach (Variable var in vars)
                    {
                        ListViewItem item = new ListViewItem(new string[] { var.Name, var.Type.Name });
                        item.Tag = var;
                        this.lvwInputMappings.Items.Add(item);
                    }
                }
            }
        }

        private void lvwInputMappings_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SavePanel2Controls();
            if ((this.lvwInputMappings.SelectedItems != null) &&
                (this.lvwInputMappings.SelectedItems.Count == 1))
            {
                ListViewItem item = this.lvwInputMappings.SelectedItems[0];
                if ((item != null) && (item.Tag is Variable))
                {
                    VariableMappingEditor editor = null;
                    if ((this.splitInputMappings.Panel2.Controls.Count == 1) &&
                        (this.splitInputMappings.Panel2.Controls[0] is VariableMappingEditor))
                    {
                        editor = (VariableMappingEditor)this.splitInputMappings.Panel2.Controls[0];
                        editor.Variable = (Variable)item.Tag;
                    }

                    if (editor == null)
                    {
                        editor = new VariableMappingEditor(
                             this.component,
                             (Variable)item.Tag);
                        editor.Dock = DockStyle.Fill;
                        this.splitInputMappings.Panel2.Controls.Clear();
                        this.splitInputMappings.Panel2.Controls.Add(editor);
                    }

                    return;
                }
            }
            this.splitInputMappings.Panel2.Controls.Clear();
        }

        private void SavePanel2Controls()
        {
            if ((this.splitInputMappings.Panel2.Controls.Count == 1) &&
                (this.splitInputMappings.Panel2.Controls[0] is VariableMappingEditor))
            {
                ((VariableMappingEditor)this.splitInputMappings.Panel2.Controls[0]).Save();
            }
       }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.SavePanel2Controls();
            if (this.component is ISOSupportsValidation)
                ((ISOSupportsValidation)this.component).RunValidation();
            this.DialogResult = this.btnClose.DialogResult;
        }
    }
}