using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Data.Mappers;

namespace Softix.Orchestration.Editor.Editors.MappingEditors
{
    public partial class SingleVariablePicker : UserControl, IVariableMapperUI
    {
        public SingleVariablePicker()
        {
            InitializeComponent();
        }

        #region IVariableMapperUI Members

        private ISOComponent component = null;

        public new void Load(
            ISOModel model,
            ISOComponent component,
            Type target,
            IVariableMapper mapper)
        {
            this.component = component;

            Variable[] variables = model.GetVariablesForComponent(component);
            if ((variables != null) && (target != null))
            {
                foreach (Variable var in variables)
                {
                    if (var.Type == null)
                        continue;

                    if (var.IsArray == true)
                    {
                        // test for array
                        TreeNode varNode = new TreeNode(
                            VariableMappingEditorFactory.GetVariableDisplayName(model, var));
                        varNode.Tag = var;
                        bool addNode = false;

                        if (target == typeof(Int32))
                        {
                            #region count node

                            TreeNode countNode = new TreeNode("Count Elements");
                            countNode.Tag = "count";
                            varNode.Nodes.Add(countNode);

                            // test for selected node
                            if ((mapper != null) && (mapper.GetType() == typeof(CountElementsMapper)))
                            {
                                CountElementsMapper x = (CountElementsMapper)mapper;
                                if ((x.Variable != null) &&
                                    (x.Variable.Name == var.Name) &&
                                    (x.Variable.SourceComponentID == var.SourceComponentID))
                                    this.tvwVariables.SelectedNode = countNode;
                            }

                            #endregion
                            addNode = true;
                        }

                        if (target.IsAssignableFrom(var.Type.GetElementType()) == true)
                        {
                            #region first, last, custom nodes

                            // FIRST index node
                            TreeNode firstNode = new TreeNode("First Element in Array");
                            firstNode.Tag = "first";
                            varNode.Nodes.Add(firstNode);

                            // test for selected node
                            if ((mapper != null) && (mapper.GetType() == typeof(FirstElementMapper)))
                            {
                                FirstElementMapper x = (FirstElementMapper)mapper;
                                if ((x.Variable != null) &&
                                    (x.Variable.Name == var.Name) &&
                                    (x.Variable.SourceComponentID == var.SourceComponentID))
                                    this.tvwVariables.SelectedNode = firstNode;
                            }

                            // LAST index node
                            TreeNode lastNode = new TreeNode("Last Element in Array");
                            lastNode.Tag = "last";
                            varNode.Nodes.Add(lastNode);

                            // test for selected node
                            if ((mapper != null) && (mapper.GetType() == typeof(LastElementMapper)))
                            {
                                LastElementMapper x = (LastElementMapper)mapper;
                                if ((x.Variable != null) &&
                                    (x.Variable.Name == var.Name) &&
                                    (x.Variable.SourceComponentID == var.SourceComponentID))
                                    this.tvwVariables.SelectedNode = lastNode;
                            }

                            // CUSTOM index node
                            TreeNode indexNode = new TreeNode("Custom Index");
                            indexNode.Tag = "index";
                            varNode.Nodes.Add(indexNode);

                            // test for selected node
                            if ((mapper != null) && (mapper.GetType() == typeof(CustomIndexMapper)))
                            {
                                CustomIndexMapper x = (CustomIndexMapper)mapper;
                                this.txtArrayIndex.Tag = x.IndexMapper;
                                this.txtArrayIndex.Text = (x.IndexMapper == null)
                                    ? "0"
                                    : x.IndexMapper.ToString();
                                if ((x.Variable != null) &&
                                    (x.Variable.Name == var.Name) &&
                                    (x.Variable.SourceComponentID == var.SourceComponentID))
                                    this.tvwVariables.SelectedNode = indexNode;
                            }

                            #endregion
                            addNode = true;
                        }

                        // test if we can add this node
                        if (addNode == true)
                            this.tvwVariables.Nodes.Add(varNode);
                    }

                    // test for simple var
                    if (target.IsAssignableFrom(var.Type) == false)
                        continue;

                    TreeNode singleVarNode = new TreeNode(
                        VariableMappingEditorFactory.GetVariableDisplayName(model, var));
                    singleVarNode.Tag = var;
                    this.tvwVariables.Nodes.Add(singleVarNode);

                    if ((mapper != null) && (mapper.GetType() == typeof(SingleVariableMapper)))
                    {
                        SingleVariableMapper x = (SingleVariableMapper)mapper;
                        if ((x.Variable != null) &&
                            (x.Variable.Name == var.Name) &&
                            (x.Variable.SourceComponentID == var.SourceComponentID))
                            this.tvwVariables.SelectedNode = singleVarNode;
                    }
                }
            }
        }

        public IVariableMapper Save()
        {
            TreeNode node = this.tvwVariables.SelectedNode;
            if (node == null)
                return null;

            // test for single variable
            if ((node.Tag is Variable) && (node.Nodes.Count == 0))
            {
                Variable var = (Variable)node.Tag;
                return new SingleVariableMapper(var);
            }
            else if (node.Tag is String)
            {
                string tag = (String)node.Tag;

                // find parent variable node
                TreeNode tmpNode = node;
                while ((tmpNode != null) && ((tmpNode.Tag is Variable) == false))
                    tmpNode = tmpNode.Parent;

                if (tmpNode != null)
                {
                    Variable var = (Variable)tmpNode.Tag;
                    switch (tag)
                    {
                        case "count":
                            return new CountElementsMapper(var);
                        case "last":
                            return new LastElementMapper(var);
                        case "first":
                            return new FirstElementMapper(var);
                        case "index":
                            CustomIndexMapper mapper = new CustomIndexMapper(var);
                            mapper.IndexMapper = this.txtArrayIndex.Tag as IVariableMapper;
                            return mapper;
                    }
                }
            }
            return null;
        }

        #endregion

        private void tvwVariables_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if ((e.Node != null) && (e.Node.Nodes.Count > 0))
            {
                e.Node.TreeView.SelectedNode = e.Node.Nodes[0];
            }
            else
            {
                // set array index panel visibility
                panelArrayIndex.Visible = (e.Node == null) ? false : ((e.Node.Tag is String)
                    ? (((String)e.Node.Tag) == "index")
                    : false);
            }
        }

        private void btnPick_Click(object sender, EventArgs e)
        {
            using (VariablePicker picker = new VariablePicker(
                this.component,
                typeof(Int32),
                this.txtArrayIndex.Tag as IVariableMapper))
            {
                if (picker.ShowDialog(this) == DialogResult.OK)
                {
                    IVariableMapper mapper = picker.Mapper;
                    this.txtArrayIndex.Tag = mapper;
                    this.txtArrayIndex.Text = (mapper != null) ? mapper.ToString() : "0";
                }
            }
        }

        private void txtArrayIndex_Validating(object sender, CancelEventArgs e)
        {
            string text = this.txtArrayIndex.Text;
            if ((this.txtArrayIndex.Tag != null) && (text == this.txtArrayIndex.Tag.ToString()))
            {
                // ok
            }
            else
            {
                // try to parse int
                int val = 0;
                if (Int32.TryParse(text, out val) == false)
                    e.Cancel = true;
                else
                {
                    this.txtArrayIndex.Tag = new ConstantValueMapper(val);
                    this.txtArrayIndex.Text = this.txtArrayIndex.Tag.ToString();
                }
            }
        }
    }
}
