using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Xml;
using System.IO;
using System.IO.Compression;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization.Formatters.Soap;

using Softix.Drawing.Diagram;
using Softix.Orchestration.Editor.Diagram;
using Softix.Orchestration.Core;
using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Editor
{
    public partial class MainForm : Form
    {
        private ISOModel model = null;
        private OrchestrationDiagram diagram = null;

        public MainForm()
        {
            InitializeComponent();

            // setup components
            ComponentInfo[] infos = ComponentFactory.GetComponents();
            foreach (ComponentInfo info in infos)
            {
                TreeNode catNode = null;
                TreeNode[] tmpNodes = tvwComponents.Nodes.Find(info.Category, true);
                if ((tmpNodes != null) && (tmpNodes.Length > 0))
                    catNode = tmpNodes[0];
                if (catNode == null)
                {
                    catNode = tvwComponents.Nodes.Add(info.Category, info.Category);
                }
                TreeNode node = new TreeNode(info.Name);
                node.Tag = info;
                catNode.Nodes.Add(node);
                catNode.Expand();
            }

            // create model
            this.model = new SimpleExecutionModel();

            // setup view
            this.view.Diagram = this.diagram = new OrchestrationDiagram(this.model);
        }

        private void tvwComponents_ItemDrag(object sender, ItemDragEventArgs e)
        {
            if (e.Item is TreeNode)
            {
                TreeNode node = (TreeNode)e.Item;
                if ((node.Tag != null) && (node.Tag is ComponentInfo))
                {
                    DiagramViewDropItem gigi = new DiagramViewDropItem(node.Tag);
                    tvwComponents.DoDragDrop(gigi, DragDropEffects.Move);
                }
            }
        }

        private void toolBtnSave_Click(object sender, EventArgs e)
        {
            if (this.dlgSave.ShowDialog(this) == DialogResult.OK)
            {
                Persistance.FileStorage.SaveDiagram(
                    this.diagram,
                    this.dlgSave.FileName);
            }
        }

        private void toolBtnOpen_Click(object sender, EventArgs e)
        {
            if (this.dlgOpen.ShowDialog(this) == DialogResult.OK)
            {
                OrchestrationDiagram diagram = Persistance.FileStorage.LoadDiagram(
                    this.dlgOpen.FileName);
                this.diagram = diagram;
                this.model = this.diagram.Model;
                this.view.Diagram = this.diagram;
            }
        }

        private void toolBtnNew_Click(object sender, EventArgs e)
        {
            this.model = new SimpleExecutionModel();
            this.diagram = new OrchestrationDiagram(this.model);
            this.view.Diagram = this.diagram;
        }

        private void toolBtnRun_DropDownOpening(object sender, EventArgs e)
        {
            this.toolBtnRun.DropDownItems.Clear();

            ISOComponent[] starts = this.model.StartComponents;
            if ((starts != null) && (starts.Length > 0))
            {
                foreach (ISOComponent start in starts)
                {
                    ToolStripMenuItem item = new ToolStripMenuItem(
                        start.Name,
                        Softix.Drawing.Diagram.Resouces.Resources.DefaultComponentImage);
                    item.Tag = start;

                    this.toolBtnRun.DropDownItems.Add(item);
                }
            }
        }

        private void toolBtnRun_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            ISOComponent start = (e.ClickedItem.Tag as ISOComponent);
            if (start != null)
            {
                DataStorage storage = this.model.Run(start);
                using (RuntimeDataViewer viewer = new RuntimeDataViewer(this.model, storage))
                {
                    viewer.ShowDialog(this);
                }
            }
        }

        private void toolBtnValidate_Click(object sender, EventArgs e)
        {
            SimpleExecutionModel model = ((SimpleExecutionModel)this.model);
            model.RunValidation();

            if (model.IsValid == false)
            {
                using (ValidationBreaksViewer viewer = new ValidationBreaksViewer(model.ValidationBreaks))
                {
                    viewer.ShowDialog(this);
                }
            }
            else
                MessageBox.Show(this, "Model is valid!", "Validation", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}