using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.IO.Compression;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml;
using System.Xml.Serialization;

using Softix.Drawing.Diagram.Core;
using Softix.Orchestration.Editor.Diagram;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Editor.Persistance
{
    internal static class FileStorage
    {
        #region save diagram

        internal static void SaveDiagram(OrchestrationDiagram diagram, String fileName)
        {
            FileStorage.SaveDiagram(diagram, new FileInfo(fileName));
        }

        internal static void SaveDiagram(OrchestrationDiagram diagram, FileInfo file)
        {
            FileInfo target = file;
            if (file.Exists == true)
                target = new FileInfo(file.FullName + ".tmp");

            Stream stream = target.OpenWrite();
            try
            {
                Stream gz = new GZipStream(stream, CompressionMode.Compress);

                BinaryFormatter formatter = new BinaryFormatter();
                formatter.TypeFormat = FormatterTypeStyle.TypesWhenNeeded;
                formatter.Context = new StreamingContext(StreamingContextStates.File);
                formatter.Serialize(gz, diagram.Model);
                // stream.Close();

                XmlTextWriter xmlw = new XmlTextWriter(gz, Encoding.ASCII);
                xmlw.Formatting = Formatting.Indented;
                xmlw.Indentation = 4;
                xmlw.IndentChar = ' ';

                XmlSerializer serializer = new XmlSerializer(typeof(OrchestrationDiagram));
                serializer.Serialize(xmlw, diagram);
                // ((IXmlSerializable)this.view.Diagram).WriteXml(writer);

                xmlw.Close();
            }
            catch // (Exception ex)
            {
                stream.Close();
                if (target.Exists == true)
                    target.Delete();
            }
            finally
            {
                try { stream.Close(); }
                catch { }
                if (target.Exists == true)
                {
                    if (file.Exists == true)
                        file.Delete();
                    target.MoveTo(file.FullName);
                }
            }
        }

        #endregion

        #region load diagram

        internal static OrchestrationDiagram LoadDiagram(String fileName)
        {
            return FileStorage.LoadDiagram(new FileInfo(fileName));
        }

        internal static OrchestrationDiagram LoadDiagram(FileInfo file)
        {
            Stream gz = new GZipStream(file.OpenRead(), CompressionMode.Decompress);
            try
            {
                OrchestrationDiagram diagram = null;

                BinaryFormatter formatter = new BinaryFormatter();
                formatter.TypeFormat = FormatterTypeStyle.TypesWhenNeeded;
                formatter.Context = new StreamingContext(StreamingContextStates.File);
                diagram = new OrchestrationDiagram((ISOModel)formatter.Deserialize(gz));

                XmlTextReader reader = new XmlTextReader(gz);
                reader.ReadStartElement("OrchestrationDiagram");
                XmlSerializer serializer = new XmlSerializer(typeof(OrchestrationDiagram));
                diagram.ReadXml(reader);

                return diagram;
            }
            finally
            {
                try { gz.Close(); }
                catch { }
            }
        }

        #endregion
    }
}
