using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Softix.Orchestration.Core.Data;
using Softix.Orchestration.Core.Components;
using Softix.Orchestration.Core.Models;

namespace Softix.Orchestration.Editor
{
    public partial class RuntimeDataViewer : Form
    {
        public RuntimeDataViewer(ISOModel model, DataStorage storage)
        {
            InitializeComponent();

            foreach (Guid compID in storage.GetComponentIDs())
            {
                ISOComponent comp = model.GetComponent(compID);
                if (comp == null)
                {
                    TreeNode node = new TreeNode(compID + " - not found");
                    node.ForeColor = Color.OrangeRed;
                    this.tvwData.Nodes.Add(node);
                    continue;
                }

                TreeNode compNode = new TreeNode(comp.Name);
                VariableValue[] vars = storage.GetOutputForComponent(comp.ID);
                if ((vars == null) || (vars.Length == 0))
                {
                    TreeNode node = new TreeNode("(no data)");
                    node.ForeColor = Color.Silver;
                    compNode.Nodes.Add(node);
                }
                else
                {
                    // this has a value
                    foreach (VariableValue var in vars)
                    {
                        if (var.Value == null)
                        {
                            // is null
                            TreeNode node = new TreeNode(var.Name + " - (null)");
                            node.ForeColor = Color.Maroon;
                            compNode.Nodes.Add(node);
                        }
                        else
                        {
                            TreeNode varNode = new TreeNode(var.Name + ": " + var.Value.GetType().Name);
                            varNode.Tag = var.Value;
                            if (var.Value.GetType().IsArray == true)
                            {
                                Array array = (Array)var.Value;
                                foreach (Object obj in array)
                                {
                                    TreeNode node = new TreeNode(
                                        (obj == null) ? "(null)" : obj.ToString());
                                    if (obj == null)
                                        node.ForeColor = Color.Maroon;
                                    node.Tag = obj;
                                    varNode.Nodes.Add(node);
                                }
                            }
                            else
                            {
                                varNode.Text += " - " + var.Value;
                            }
                            compNode.Nodes.Add(varNode);
                        }
                    }
                }
                compNode.Expand();
                this.tvwData.Nodes.Add(compNode);
            }
        }

        private void tvwData_AfterSelect(object sender, TreeViewEventArgs e)
        {
            this.txtRaw.Text = "";
            if ((e.Node != null) && (e.Node.Tag != null))
            {
                Object val = e.Node.Tag;
                if (val.GetType().IsArray == true)
                {
                    Array array = (Array)val;
                    for (int i = 0; i < array.Length; i++)
                    {
                        if (this.txtRaw.Text.Length > 0)
                            this.txtRaw.Text += Environment.NewLine;
                        Object tmpVal = array.GetValue(i);
                        this.txtRaw.Text += i + ": " + (
                            (tmpVal == null) ? "(null)" : tmpVal.ToString()
                        );
                    }
                }
                else
                {
                    this.txtRaw.Text = val.ToString();
                }
            }
        }
    }
}